@@ -5,10 +5,16 @@ list see the documentation:
 https://www.sphinx-doc.org/en/master/usage/configuration.html
 """
 
-import configparser
+import json
 import os
 import sys
 
+# tomllib is only in the stdlib from 3.11+
+if sys.version_info >= (3, 11):
+    import tomllib
+else:  # pragma: no cover
+    import toml as tomllib
+
 # -- Path setup --------------------------------------------------------------
 
 # If extensions (or modules to document with autodoc) are in another directory,
@@ -19,14 +25,14 @@ sys.path.append(os.path.abspath("./_ext"))
 
 # Get the global config info as currently stated
 # (we use the config file to avoid actually loading any python here)
-config = configparser.ConfigParser()
-config.read(["../../setup.cfg"])
-stable_version = config.get("sqlfluff_docs", "stable_version")
+with open("../../pyproject.toml", "rb") as config_file:
+    config = tomllib.load(config_file)
+stable_version = config.get("tool.sqlfluff_docs", "stable_version")
 
 # -- Project information -----------------------------------------------------
 
 project = "SQLFluff"
-copyright = "2023, Alan Cruickshank"
+copyright = "2024, Alan Cruickshank"
 author = "Alan Cruickshank"
 
 # The full version, including alpha/beta/rc tags
@@ -60,7 +66,7 @@ templates_path = ["_templates"]
 exclude_patterns = [
     # Exclude the partials folder, which is made up of files intended
     # to be included in others.
-    "partials",
+    "_partials",
 ]
 
 # Master doc
@@ -105,11 +111,65 @@ html_theme_options = {
 # -- Options for redirects ---------------------------------------------
 # https://documatt.gitlab.io/sphinx-reredirects/usage.html
 
+# Load the rule lists to generate rule permalinks
+with open("_partials/rule_list.json", "r") as rule_file:
+    rule_list = json.load(rule_file)
+
 redirects = {
-    # There's an old link to /indentation in config files.
-    # That should point to the layout section now.
-    "indentation": "layout.html#configuring-indent-locations",
-    "architecture": "internals.html#architecture",
+    # Where there are references to the docs in any of the codebase (whether in
+    # places like the README or in error messages), they should all reference
+    # a perma link (to redirect). This ensures we can support a consistent
+    # link location even if the docs move around.
+    "perma/layout": "../configuration/layout.html",
+    "perma/indent_locations": (
+        "../configuration/layout.html#configuring-indent-locations"
+    ),
+    "perma/hanging_indents": "../configuration/layout.html#hanging-indents",
+    "perma/layout_spacing": (
+        "../configuration/layout.html#configuring-layout-and-spacing"
+    ),
+    "perma/configuration": "../configuration/index.html",
+    "perma/dbt": "../configuration/templating/dbt.html",
+    "perma/cli": "../reference/cli.html",
+    "perma/rules": "../reference/rules.html",
+    "perma/dialects": "../reference/dialects.html",
+    "perma/architecture": "../guides/contributing/architecture.html",
+    "perma/rule_disabling": (
+        "../configuration/rule_configuration.html#enabling-and-disabling-rules"
+    ),
+    "perma/internals": "../reference/internals/index.html",
+    "perma/modularity": "../why_sqlfluff.html#modularity",
+    "perma/indentation": "../configuration/layout.html#configuring-indent-locations",
+    "perma/releasenotes": "../reference/releasenotes.html",
+    "perma/why": "../why_sqlfluff.html",
+    "perma/plugin_dev": "../guides/contributing/plugins.html",
+    "perma/plugin_guide": "../guides/setup/developing_custom_rules.html",
+    "perma/variables": "../configuration/templating/index.html",
+    "perma/python_templating": "../configuration/templating/python.html",
+    "perma/guides": "../guides/index.html",
+    "perma/contribute_dialect_keywords": (
+        "../guides/contributing/dialect.html#dialect_keywords"
+    ),
+    # Add permalinks for rule codes
+    **{
+        f"perma/rule/{code}": (
+            f"../../reference/rules.html#sqlfluff.rules.sphinx.Rule_{code}"
+        )
+        for code, _ in rule_list
+    },
+    # These are legacy links which used to exist in different parts of the
+    # SQLFluff code base, and which we continue to support so those links
+    # aren't dead ends. They should redirect to permalinks.
+    "indentation": "perma/indentation.html",
+    "architecture": "perma/architecture.html",
+    "dialects": "perma/dialects.html",
+    "internals": "perma/internals.html",
+    "layout": "perma/layout.html",
+    "releasenotes": "perma/releasenotes.html",
+    "realworld": "perma/why.html",
+    # This is a legacy link to support older versions of the VSCode plugin.
+    # https://github.com/sqlfluff/vscode-sqlfluff/blob/master/src/features/providers/linter/actions/hover.ts
+    "rules": "perma/rules.html",
 }
 
 
