@@ -1,17 +1,13 @@
 """Sqlfluff is a SQL linter for humans."""
+
 import sys
+from importlib import metadata
 
 import pytest
 
 # Expose the public API.
 from sqlfluff.api import fix, lint, list_dialects, list_rules, parse
 
-# Import metadata (using importlib_metadata backport for python versions <3.8)
-if sys.version_info >= (3, 8):
-    from importlib import metadata
-else:  # pragma: no cover
-    import importlib_metadata as metadata
-
 __all__ = (
     "lint",
     "fix",
@@ -27,9 +23,9 @@ __version__ = metadata.version("sqlfluff")
 if sys.version_info[0] < 3:
     raise Exception("Sqlfluff does not support Python 2. Please upgrade to Python 3.")
 # Check minor python version
-elif sys.version_info[1] < 7:
+elif sys.version_info[1] < 8:
     raise Exception(
-        "Sqlfluff %s only supports Python 3.7 and beyond. "
+        "Sqlfluff %s only supports Python 3.8 and beyond. "
         "Use an earlier version of sqlfluff or a later version of Python" % __version__
     )
 
