@@ -1,16 +1,15 @@
 """Definitions of grammars."""
 
-from sqlfluff.core.parser.grammar.base import Ref, Anything, Nothing
 from sqlfluff.core.parser.grammar.anyof import (
     AnyNumberOf,
     AnySetOf,
     OneOf,
     OptionallyBracketed,
 )
-from sqlfluff.core.parser.grammar.delimited import Delimited
-from sqlfluff.core.parser.grammar.greedy import GreedyUntil, StartsWith
-from sqlfluff.core.parser.grammar.sequence import Sequence, Bracketed
+from sqlfluff.core.parser.grammar.base import Anything, Nothing, Ref
 from sqlfluff.core.parser.grammar.conditional import Conditional
+from sqlfluff.core.parser.grammar.delimited import Delimited
+from sqlfluff.core.parser.grammar.sequence import Bracketed, Sequence
 
 __all__ = (
     "Ref",
@@ -21,8 +20,6 @@ __all__ = (
     "OneOf",
     "OptionallyBracketed",
     "Delimited",
-    "GreedyUntil",
-    "StartsWith",
     "Sequence",
     "Bracketed",
     "Conditional",
