@@ -28,7 +28,7 @@ if TYPE_CHECKING:  # pragma: no cover
 
 
 def cached_method_for_parse_context(
-    func: Callable[[Any, ParseContext, Optional[Tuple[str]]], SimpleHintType]
+    func: Callable[[Any, ParseContext, Optional[Tuple[str]]], SimpleHintType],
 ) -> Callable[..., SimpleHintType]:
     """A decorator to cache the output of this method for a given parse context.
 
@@ -215,11 +215,18 @@ class BaseGrammar(Matchable):
             for k in self.equality_kwargs
         )
 
+    def __ne__(self, other: Any) -> bool:  # pragma: no cover
+        """Two grammars are not equal if their elements or types are not equal.
+
+        NOTE: This is to handle some conflicts with dunder methods in mypyc.
+        """
+        return not self.__eq__(other)
+
     def copy(
         self: T,
         insert: Optional[List[Matchable]] = None,
         at: Optional[int] = None,
-        before: Optional[Any] = None,
+        before: Optional[Matchable] = None,
         remove: Optional[List[Matchable]] = None,
         terminators: List[Union[str, Matchable]] = [],
         replace_terminators: bool = False,
