@@ -1,9 +1,8 @@
 """Helpers for the parser module."""
 
-from typing import Tuple, List, Any, Iterator, TYPE_CHECKING
+from typing import TYPE_CHECKING, Tuple
 
 from sqlfluff.core.errors import SQLParseError
-from sqlfluff.core.string_helpers import curtail_string
 
 if TYPE_CHECKING:
     from sqlfluff.core.parser.segments import BaseSegment  # pragma: no cover
@@ -14,11 +13,6 @@ def join_segments_raw(segments: Tuple["BaseSegment", ...]) -> str:
     return "".join(s.raw for s in segments)
 
 
-def join_segments_raw_curtailed(segments: Tuple["BaseSegment", ...], length=20) -> str:
-    """Make a string up to a certain length from an iterable of segments."""
-    return curtail_string(join_segments_raw(segments), length=length)
-
-
 def check_still_complete(
     segments_in: Tuple["BaseSegment", ...],
     matched_segments: Tuple["BaseSegment", ...],
@@ -31,14 +25,14 @@ def check_still_complete(
     if initial_str != current_str:  # pragma: no cover
         segment = unmatched_segments[0] if unmatched_segments else None
         raise SQLParseError(
-            f"Could not parse: {current_str}",
+            f"Parse completeness check fail: {current_str!r} != {initial_str!r}",
             segment=segment,
         )
     return True
 
 
 def trim_non_code_segments(
-    segments: Tuple["BaseSegment", ...]
+    segments: Tuple["BaseSegment", ...],
 ) -> Tuple[
     Tuple["BaseSegment", ...], Tuple["BaseSegment", ...], Tuple["BaseSegment", ...]
 ]:
@@ -62,25 +56,3 @@ def trim_non_code_segments(
             post_idx -= 1
 
     return segments[:pre_idx], segments[pre_idx:post_idx], segments[post_idx:]
-
-
-def iter_indices(seq: List, val: Any) -> Iterator[int]:
-    """Iterate all indices in a list that val occurs at.
-
-    Args:
-        seq (list): A list to look for indices in.
-        val: What to look for.
-
-    Yields:
-        int: The index of val in seq.
-
-    Examples:
-        The function works like str.index() but iterates all
-        the results rather than returning the first.
-
-        >>> print([i for i in iter_indices([1, 0, 2, 3, 2], 2)])
-        [2, 4]
-    """
-    for idx, el in enumerate(seq):
-        if el == val:
-            yield idx
