@@ -4,7 +4,7 @@ Matchable objects which return individual segments.
 """
 
 from abc import abstractmethod
-from typing import Any, Collection, Dict, Optional, Sequence, Tuple, Type
+from typing import Any, Callable, Collection, Dict, Optional, Sequence, Tuple, Type
 from uuid import uuid4
 
 import regex
@@ -31,6 +31,7 @@ class BaseParser(Matchable):
         optional: bool = False,
         # The following kwargs are passed on to the segment:
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ) -> None:
         self.raw_class = raw_class
         # Store instance_types rather than just type to allow
@@ -39,6 +40,7 @@ class BaseParser(Matchable):
         self._instance_types: Tuple[str, ...] = (type or raw_class.type,)
         self.optional = optional
         self._trim_chars = trim_chars
+        self.casefold = casefold
         # Generate a cache key
         self._cache_key = uuid4().hex
 
@@ -53,20 +55,26 @@ class BaseParser(Matchable):
         """Return whether this element is optional."""
         return self.optional
 
-    def _match_at(self, idx: int) -> MatchResult:
-        """Construct a MatchResult at a given index.
-
-        This is a helper function for reuse by other parsers.
-        """
+    def segment_kwargs(self) -> Dict[str, Any]:
+        """Generates the segment_kwargs package for generating a matched segment."""
         segment_kwargs: Dict[str, Any] = {}
         if self._instance_types:
             segment_kwargs["instance_types"] = self._instance_types
         if self._trim_chars:
             segment_kwargs["trim_chars"] = self._trim_chars
+        if self.casefold:
+            segment_kwargs["casefold"] = self.casefold
+        return segment_kwargs
+
+    def _match_at(self, idx: int) -> MatchResult:
+        """Construct a MatchResult at a given index.
+
+        This is a helper function for reuse by other parsers.
+        """
         return MatchResult(
             matched_slice=slice(idx, idx + 1),
             matched_class=self.raw_class,
-            segment_kwargs=segment_kwargs,
+            segment_kwargs=self.segment_kwargs(),
         )
 
 
@@ -80,6 +88,7 @@ class TypedParser(BaseParser):
         type: Optional[str] = None,
         optional: bool = False,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ) -> None:
         """Initialize a new instance of the class.
 
@@ -89,6 +98,7 @@ class TypedParser(BaseParser):
             type (Optional[str]): The type of the instance.
             optional (bool): Whether the instance is optional.
             trim_chars (Optional[Tuple[str, ...]]): The characters to trim.
+            casefold: (Optional[Callable[[str],str]]): The default casing used.
 
         Returns:
             None
@@ -102,6 +112,7 @@ class TypedParser(BaseParser):
             raw_class=raw_class,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
         # NOTE: We override the instance types after initialising the base
         # class. We want to ensure that re-matching is possible by ensuring that
@@ -167,6 +178,7 @@ class StringParser(BaseParser):
         type: Optional[str] = None,
         optional: bool = False,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         self.template = template.upper()
         # Create list version upfront to avoid recreating it multiple times.
@@ -176,6 +188,7 @@ class StringParser(BaseParser):
             type=type,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
 
     def __repr__(self) -> str:
@@ -217,6 +230,7 @@ class MultiStringParser(BaseParser):
         type: Optional[str] = None,
         optional: bool = False,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         self.templates = {template.upper() for template in templates}
         # Create list version upfront to avoid recreating it multiple times.
@@ -226,6 +240,7 @@ class MultiStringParser(BaseParser):
             type=type,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
 
     def __repr__(self) -> str:
@@ -268,6 +283,7 @@ class RegexParser(BaseParser):
         optional: bool = False,
         anti_template: Optional[str] = None,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         # Store the optional anti-template
         self.template = template
@@ -280,13 +296,14 @@ class RegexParser(BaseParser):
             type=type,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
 
     def __repr__(self) -> str:
         return f"<RegexParser: {self.template!r}>"
 
     def simple(
-        cls, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
+        self, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
     ) -> None:
         """Does this matcher support a uppercase hash matching route?
 
