@@ -1,12 +1,14 @@
 """Indent and Dedent classes."""
 
+from typing import List, Optional, Sequence, Tuple
 from uuid import UUID
 
-from sqlfluff.core.parser.markers import PositionMarker
-from sqlfluff.core.parser.match_wrapper import match_wrapper
-from sqlfluff.core.parser.segments.raw import RawSegment, SourceFix
 from sqlfluff.core.parser.context import ParseContext
-from typing import Optional, List
+from sqlfluff.core.parser.markers import PositionMarker
+from sqlfluff.core.parser.match_result import MatchResult
+from sqlfluff.core.parser.segments.base import BaseSegment
+from sqlfluff.core.parser.segments.raw import RawSegment, SourceFix
+from sqlfluff.core.templaters.base import TemplatedFile
 
 
 class MetaSegment(RawSegment):
@@ -16,32 +18,38 @@ class MetaSegment(RawSegment):
     _is_code = False
     _template = "<unset>"
     indent_val = 0
+    # Implicit indents are to be considered _taken_ unless
+    # closed on the same line.
+    is_implicit = False
     is_meta = True
+    _preface_modifier = "[META] "
 
     def __init__(
         self,
+        pos_marker: Optional[PositionMarker] = None,
         is_template: bool = False,
         block_uuid: Optional[UUID] = None,
-        *args,
-        **kwargs,
+        source_fixes: Optional[List[SourceFix]] = None,
     ):
         """Constructor for MetaSegment.
 
         Args:
-            is_template (:obj:`bool`): A flag to indicate whether
-                this meta segment is related to a templated section.
-                This allows proper handling.
-            block_uuid (:obj:`UUID`): A reference to link together
-                markers which refer to the same structure in a
-                template (e.g. the beginning and end of an if
-                statement).
+            pos_marker (:obj:`PositionMarker`, optional): The position
+                of the segment.
+            is_template (:obj:`bool`, optional): A flag to indicate whether
+                this meta segment is related to a templated section. This
+                allows proper handling.
+            block_uuid (:obj:`UUID`, optional): A reference to link together
+                markers which refer to the same structure in a template
+                (e.g. the beginning and end of an if statement).
+            source_fixes: (:obj:`list` of :obj:`SourceFix`, optional): A
+                list of any source fixes to apply to this segment.
         """
-        super().__init__(*args, **kwargs)
+        super().__init__(pos_marker=pos_marker, source_fixes=source_fixes)
         self.is_template = is_template
         self.block_uuid = block_uuid
 
-    @staticmethod
-    def _suffix():
+    def _suffix(self) -> str:
         """Return any extra output required at the end when logging.
 
         Meta classes have not much to say here so just stay blank.
@@ -49,8 +57,9 @@ class MetaSegment(RawSegment):
         return ""
 
     @classmethod
-    @match_wrapper()
-    def match(cls, segments, parse_context):  # pragma: no cover
+    def match(
+        cls, segments: Sequence["BaseSegment"], idx: int, parse_context: ParseContext
+    ) -> MatchResult:  # pragma: no cover
         """This will never be called. If it is then we're using it wrong."""
         raise NotImplementedError(
             "{} has no match method, it should only be used in a Sequence!".format(
@@ -59,7 +68,9 @@ class MetaSegment(RawSegment):
         )
 
     @classmethod
-    def simple(cls, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
+    def simple(
+        cls, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
+    ) -> None:
         """Does this matcher support an uppercase hash matching route?
 
         This should be true if the MATCH grammar is simple. Most more
@@ -103,6 +114,27 @@ class Indent(MetaSegment):
     type = "indent"
     indent_val = 1
 
+    def _suffix(self) -> str:
+        """If present, output the block uuid."""
+        return f"[Block: {self.block_uuid.hex[:6]!r}]" if self.block_uuid else ""
+
+
+class ImplicitIndent(Indent):
+    """A variant on the indent, that is considered *taken* unless closed in line.
+
+    This is primarily for facilitating constructions which behave a little
+    like hanging indents, without the complicated indentation spacing.
+
+    .. code-block:: sql
+        SELECT *
+        FROM foo
+        WHERE a  -- The theoretical indent between WHERE and "a" is implicit.
+            AND b
+    """
+
+    _preface_modifier = "[META] (implicit) "
+    is_implicit = True
+
 
 class Dedent(Indent):
     """A segment which is empty but indicates where an dedent should be.
@@ -144,7 +176,8 @@ class TemplateSegment(MetaSegment):
         block_uuid: Optional[UUID] = None,
     ):
         """Initialise a placeholder with the source code embedded."""
-        if not source_str:  # pragma: no cover
+        # NOTE: Empty string is ok, None is not.
+        if source_str is None:  # pragma: no cover
             raise ValueError("Cannot instantiate TemplateSegment without a source_str.")
         self.source_str = source_str
         self.block_type = block_type
@@ -153,25 +186,60 @@ class TemplateSegment(MetaSegment):
             pos_marker=pos_marker, source_fixes=source_fixes, block_uuid=block_uuid
         )
 
-    def _suffix(self):
+    def _suffix(self) -> str:
         """Also output what it's a placeholder for."""
-        return f"[Type: {self.block_type!r}, Raw: {self.source_str!r}]"
+        return (
+            f"[Type: {self.block_type!r}, Raw: {self.source_str!r}"
+            + (f", Block: {self.block_uuid.hex[:6]!r}" if self.block_uuid else "")
+            + "]"
+        )
 
-    def to_tuple(self, code_only=False, show_raw=False, include_meta=False):
+    @classmethod
+    def from_slice(
+        cls,
+        source_slice: slice,
+        templated_slice: slice,
+        block_type: str,
+        templated_file: TemplatedFile,
+        block_uuid: Optional[UUID] = None,
+    ) -> "TemplateSegment":
+        """Construct template segment from slice of a source file."""
+        pos_marker = PositionMarker(
+            source_slice,
+            templated_slice,
+            templated_file,
+        )
+        return cls(
+            pos_marker=pos_marker,
+            source_str=templated_file.source_str[source_slice],
+            block_type=block_type,
+            block_uuid=block_uuid,
+        )
+
+    def to_tuple(
+        self,
+        code_only: bool = False,
+        show_raw: bool = False,
+        include_meta: bool = False,
+    ) -> Tuple[str, str]:
         """Return a tuple structure from this segment.
 
         Unlike most segments, we return the _source_ content for placeholders
         if viewing metas is allowed. This allows verification of the content
         of those placeholders for inspection or debugging.
+
+        NOTE: This method does not use the `include_meta` argument. This method
+        relies on any parent segment to do filtering associated with whether to
+        include or not include meta segments.
         """
-        if include_meta:
-            return (self.get_type(), self.source_str)
-        else:  # pragma: no cover TODO?
-            return (self.get_type(), self.raw)
+        return (self.get_type(), self.source_str)
 
     def edit(
-        self, raw: Optional[str] = None, source_fixes: Optional[List[SourceFix]] = None
-    ):
+        self,
+        raw: Optional[str] = None,
+        source_fixes: Optional[List[SourceFix]] = None,
+        source_str: Optional[str] = None,
+    ) -> MetaSegment:
         """Create a new segment, with exactly the same position but different content.
 
         Returns:
@@ -186,10 +254,17 @@ class TemplateSegment(MetaSegment):
             raise ValueError(
                 "Cannot set raw of a template placeholder!"
             )  # pragma: no cover
+
+        if source_fixes or self.source_fixes:
+            sf = (source_fixes or []) + (self.source_fixes + [])
+        else:  # pragma: no cover
+            # There's _usually_ a source fix if we're editing a templated
+            # segment - but not necessarily guaranteed.
+            sf = None
         return self.__class__(
             pos_marker=self.pos_marker,
-            source_str=self.source_str,
+            source_str=source_str if source_str is not None else self.source_str,
             block_type=self.block_type,
-            source_fixes=source_fixes or self.source_fixes,
+            source_fixes=sf,
             block_uuid=self.block_uuid,
         )
