@@ -8,8 +8,14 @@ from sqlfluff.core.parser import (
     AnyNumberOf,
     BaseSegment,
     Bracketed,
+    CodeSegment,
+    Dedent,
     Delimited,
-    TypedParser,
+    IdentifierSegment,
+    Indent,
+    KeywordSegment,
+    LiteralSegment,
+    Matchable,
     Nothing,
     OneOf,
     OptionallyBracketed,
@@ -20,9 +26,8 @@ from sqlfluff.core.parser import (
     StringLexer,
     StringParser,
     SymbolSegment,
+    TypedParser,
 )
-from sqlfluff.core.parser.grammar.anyof import AnySetOf
-from sqlfluff.core.parser.segments.raw import CodeSegment, KeywordSegment, RawSegment
 from sqlfluff.dialects import dialect_ansi as ansi
 from sqlfluff.dialects.dialect_athena_keywords import (
     athena_reserved_keywords,
@@ -31,7 +36,17 @@ from sqlfluff.dialects.dialect_athena_keywords import (
 
 ansi_dialect = load_raw_dialect("ansi")
 
-athena_dialect = ansi_dialect.copy_as("athena")
+athena_dialect = ansi_dialect.copy_as(
+    "athena",
+    formatted_name="AWS Athena",
+    docstring="""**Default Casing**: ``lowercase``
+
+**Quotes**: String Literals: ``''``, ``""`` or |back_quotes|,
+Identifiers: ``""`` or |back_quotes|
+
+The dialect for `Athena <https://aws.amazon.com/athena/>`_
+on Amazon Web Services (AWS).""",
+)
 
 athena_dialect.sets("unreserved_keywords").update(athena_unreserved_keywords)
 athena_dialect.sets("reserved_keywords").update(athena_reserved_keywords)
@@ -44,7 +59,7 @@ athena_dialect.insert_lexer_matchers(
     before="like_operator",
 )
 
-athena_dialect.sets("angle_bracket_pairs").update(
+athena_dialect.bracket_sets("angle_bracket_pairs").update(
     [
         ("angle", "StartAngleBracketSegment", "EndAngleBracketSegment", False),
     ]
@@ -73,12 +88,10 @@ athena_dialect.add(
         Ref("EqualsSegment"),
         Ref("QuotedLiteralSegment"),
     ),
-    LocationGrammar=Sequence("LOCATION", Ref("S3UrlGrammar")),
-    S3UrlGrammar=RegexParser(r"^'s3://.*", RawSegment),
+    LocationGrammar=Sequence("LOCATION", Ref("QuotedLiteralSegment")),
     BracketedPropertyListGrammar=Bracketed(Delimited(Ref("PropertyGrammar"))),
     CTASPropertyGrammar=Sequence(
         OneOf(
-            "external_location",
             "format",
             "partitioned_by",
             "bucketed_by",
@@ -86,12 +99,51 @@ athena_dialect.add(
             "write_compression",
             "orc_compression",
             "parquet_compression",
+            "compression_level",
             "field_delimiter",
+            "is_external",
+            "table_type",
+            "external_location",
         ),
         Ref("EqualsSegment"),
         Ref("LiteralGrammar"),
     ),
-    BracketedCTASPropertyGrammar=Bracketed(Delimited(Ref("CTASPropertyGrammar"))),
+    CTASIcebergPropertyGrammar=Sequence(
+        OneOf(
+            "format",
+            "partitioned_by",
+            "bucketed_by",
+            "bucket_count",
+            "write_compression",
+            "orc_compression",
+            "parquet_compression",
+            "compression_level",
+            "field_delimiter",
+            "is_external",
+            "table_type",
+            # Iceberg-specific properties
+            "location",
+            "partitioning",
+            "vacuum_max_snapshot_age_seconds",
+            "vacuum_min_snapshots_to_keep",
+            "optimize_rewrite_min_data_file_size_bytes",
+            "optimize_rewrite_max_data_file_size_bytes",
+            "optimize_rewrite_data_file_threshold",
+            "optimize_rewrite_delete_file_threshold",
+        ),
+        Ref("EqualsSegment"),
+        Ref("LiteralGrammar"),
+    ),
+    BracketedCTASPropertyGrammar=Bracketed(
+        OneOf(
+            Delimited(
+                Ref("CTASPropertyGrammar"),
+            ),
+            Delimited(
+                Ref("CTASIcebergPropertyGrammar"),
+            ),
+        ),
+    ),
     UnloadPropertyGrammar=Sequence(
         OneOf(
             "format",
@@ -158,29 +210,50 @@ athena_dialect.add(
     ),
     BackQuotedIdentifierSegment=TypedParser(
         "back_quote",
-        ansi.LiteralSegment,
+        IdentifierSegment,
         type="quoted_identifier",
+        casefold=str.lower,
     ),
-    DatetimeWithTZSegment=Sequence(OneOf("TIMESTAMP", "TIME"), "WITH", "TIME", "ZONE"),
 )
 
 athena_dialect.replace(
+    LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
+        insert=[
+            Ref("ParameterSegment"),
+        ]
+    ),
+    AccessorGrammar=Sequence(
+        AnyNumberOf(
+            Ref("ArrayAccessorSegment"),
+            optional=True,
+        ),
+        AnyNumberOf(
+            Sequence(
+                Ref("ObjectReferenceDelimiterGrammar"),
+                Ref("ObjectReferenceSegment"),
+            ),
+            optional=True,
+        ),
+    ),
     QuotedLiteralSegment=OneOf(
-        TypedParser("single_quote", ansi.LiteralSegment, type="quoted_literal"),
-        TypedParser("double_quote", ansi.LiteralSegment, type="quoted_literal"),
-        TypedParser("back_quote", ansi.LiteralSegment, type="quoted_literal"),
+        TypedParser("single_quote", LiteralSegment, type="quoted_literal"),
+        TypedParser("double_quote", LiteralSegment, type="quoted_literal"),
+        TypedParser("back_quote", LiteralSegment, type="quoted_literal"),
     ),
-    SimpleArrayTypeGrammar=Ref.keyword("ARRAY"),
     TrimParametersGrammar=Nothing(),
     NakedIdentifierSegment=SegmentGenerator(
         # Generate the anti template from the set of reserved keywords
         lambda dialect: RegexParser(
-            r"([_]+|[A-Z0-9_]*[A-Z][A-Z0-9_]*)",
-            ansi.IdentifierSegment,
+            r"[A-Z0-9_]*[A-Z_][A-Z0-9_]*",
+            IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+            casefold=str.lower,
         )
     ),
+    QuotedIdentifierSegment=TypedParser(
+        "double_quote", IdentifierSegment, type="quoted_identifier", casefold=str.lower
+    ),
     SingleIdentifierGrammar=ansi_dialect.get_grammar("SingleIdentifierGrammar").copy(
         insert=[
             Ref("BackQuotedIdentifierSegment"),
@@ -194,16 +267,100 @@ athena_dialect.replace(
         # Add arrow operators for functions (e.g. filter)
         Ref("RightArrowOperator"),
     ),
+    PostFunctionGrammar=ansi_dialect.get_grammar("PostFunctionGrammar").copy(
+        # UNNEST can optionally have a WITH ORDINALITY clause
+        insert=[
+            Sequence("WITH", "ORDINALITY", optional=True),
+        ]
+    ),
+    AlterTableDropColumnGrammar=Sequence(
+        "DROP",
+        Ref.keyword("COLUMN"),
+        Ref("SingleIdentifierGrammar"),
+    ),
 )
 
 
-class PrimitiveTypeSegment(BaseSegment):
-    """Primitive data types.
+class ArrayTypeSegment(ansi.ArrayTypeSegment):
+    """Prefix for array literals specifying the type."""
 
-    Since DDL is based on Hive and DML based on Prestodb this class has
-    primitives that may not work on specific situations
-    - Hive: https://cwiki.apache.org/confluence/display/hive/languagemanual+types
-    - PrestoDb: https://prestodb.io/docs/0.217/language/types.html
+    type = "array_type"
+    match_grammar = Sequence(
+        "ARRAY",
+        Ref("ArrayTypeSchemaSegment", optional=True),
+    )
+
+
+class ArrayTypeSchemaSegment(ansi.ArrayTypeSegment):
+    """Prefix for array literals specifying the type."""
+
+    type = "array_type_schema"
+    match_grammar = Bracketed(
+        Ref("DatatypeSegment"),
+        bracket_pairs_set="angle_bracket_pairs",
+        bracket_type="angle",
+    )
+
+
+class MapTypeSegment(BaseSegment):
+    """Expression to construct a MAP datatype."""
+
+    type = "map_type"
+    match_grammar = Sequence(
+        "MAP",
+        Ref("MapTypeSchemaSegment", optional=True),
+    )
+
+
+class MapTypeSchemaSegment(BaseSegment):
+    """Expression to construct the schema of a MAP datatype."""
+
+    type = "map_type_schema"
+    match_grammar = Bracketed(
+        Sequence(
+            Ref("PrimitiveTypeSegment"),
+            Ref("CommaSegment"),
+            Ref("DatatypeSegment"),
+        ),
+        bracket_pairs_set="angle_bracket_pairs",
+        bracket_type="angle",
+    )
+
+
+class StructTypeSegment(ansi.StructTypeSegment):
+    """Expression to construct a STRUCT datatype."""
+
+    match_grammar = Sequence(
+        "STRUCT",
+        Ref("StructTypeSchemaSegment", optional=True),
+    )
+
+
+class StructTypeSchemaSegment(BaseSegment):
+    """Expression to construct the schema of a STRUCT datatype."""
+
+    type = "struct_type_schema"
+    match_grammar = Bracketed(
+        Delimited(
+            Sequence(
+                Ref("NakedIdentifierSegment"),
+                Ref("ColonSegment"),
+                Ref("DatatypeSegment"),
+                Ref("CommentGrammar", optional=True),
+            ),
+            bracket_pairs_set="angle_bracket_pairs",
+        ),
+        bracket_pairs_set="angle_bracket_pairs",
+        bracket_type="angle",
+    )
+
+
+class PrimitiveTypeSegment(BaseSegment):
+    """Support Athena subset of Hive types.
+
+    Primary Source: https://docs.aws.amazon.com/athena/latest/ug/data-types.html
+    Additional Details:
+        https://cwiki.apache.org/confluence/display/Hive/LanguageManual+Types
     """
 
     type = "primitive_type"
@@ -211,123 +368,71 @@ class PrimitiveTypeSegment(BaseSegment):
         "BOOLEAN",
         "TINYINT",
         "SMALLINT",
-        "INTEGER",
-        "INT",
+        "INTEGER",  # used in DML queries
+        "INT",  # used in DDL queries
         "BIGINT",
-        "REAL",
-        "FLOAT",
-        Sequence("DOUBLE", Ref.keyword("PRECISION", optional=True)),
+        "DOUBLE",
+        "FLOAT",  # used in DDL
+        "REAL",  # used "in SQL functions like SELECT CAST"
         Sequence(
-            "DECIMAL",
-            Bracketed(
-                Ref("NumericLiteralSegment"),
-                Ref("CommaSegment"),
-                Ref("NumericLiteralSegment"),
-                optional=True,
-            ),
+            OneOf("DECIMAL", "CHAR", "VARCHAR"),
+            Ref("BracketedArguments", optional=True),
         ),
-        "NUMERIC",
         "STRING",
-        "VARCHAR",
-        "CHAR",
-        "VARBINARY",
-        "JSON",
+        "BINARY",
         "DATE",
         "TIMESTAMP",
-        "INTERVAL",
+        "VARBINARY",
+        "JSON",
         "TIME",
         "IPADDRESS",
         "HyperLogLog",
         "P4HyperLogLog",
-        "QDigest",
     )
 
 
 class DatatypeSegment(BaseSegment):
-    """Data types."""
+    """Support complex Athena data types.
+
+    Complex data types are typically used in either DDL statements or as
+    the target type in casts.
+    """
 
     type = "data_type"
     match_grammar = OneOf(
         Ref("PrimitiveTypeSegment"),
-        Sequence(
-            "ARRAY",
-            Bracketed(
-                Ref("DatatypeSegment"),
-                bracket_pairs_set="angle_bracket_pairs",
-                bracket_type="angle",
-            ),
-        ),
-        Sequence(
-            "MAP",
-            Bracketed(
-                Sequence(
-                    Ref("PrimitiveTypeSegment"),
-                    Ref("CommaSegment"),
-                    Ref("DatatypeSegment"),
-                ),
-                bracket_pairs_set="angle_bracket_pairs",
-                bracket_type="angle",
-            ),
-        ),
-        Sequence(
-            "STRUCT",
-            Bracketed(
-                Delimited(
-                    Sequence(
-                        Ref("NakedIdentifierSegment"),
-                        Ref("ColonSegment"),
-                        Ref("DatatypeSegment"),
-                        Ref("CommentGrammar", optional=True),
-                    ),
-                    bracket_pairs_set="angle_bracket_pairs",
-                ),
-                bracket_pairs_set="angle_bracket_pairs",
-                bracket_type="angle",
-            ),
-        ),
-        # Only hive
-        Sequence(
-            "UNIONTYPE",
-            Bracketed(
-                Delimited(
-                    Ref("DatatypeSegment"), bracket_pairs_set="angle_bracket_pairs"
-                ),
-                bracket_pairs_set="angle_bracket_pairs",
-                bracket_type="angle",
-            ),
-        ),
-        # Only PrestoDb
+        Ref("StructTypeSegment"),
+        Ref("ArrayTypeSegment"),
+        Ref("MapTypeSegment"),
         Sequence(
             "ROW",
             Bracketed(
                 Delimited(
-                    AnySetOf(
-                        Sequence(Ref("NakedIdentifierSegment"), Ref("DatatypeSegment")),
+                    AnyNumberOf(
+                        Sequence(
+                            Ref("NakedIdentifierSegment"),
+                            Ref("DatatypeSegment"),
+                        ),
                         Ref("LiteralGrammar"),
                     )
                 )
             ),
         ),
-        # array types
-        OneOf(
-            AnyNumberOf(
-                Bracketed(
-                    Ref("ExpressionSegment", optional=True), bracket_type="square"
-                )
-            ),
-            Ref("SimpleArrayTypeGrammar"),
-            Sequence(Ref("SimpleArrayTypeGrammar"), Ref("ArrayLiteralSegment")),
-            optional=True,
-        ),
-        Ref("DatetimeWithTZSegment"),
+        Ref("TimeWithTZGrammar"),
     )
 
 
 class StatementSegment(ansi.StatementSegment):
     """Overriding StatementSegment to allow for additional segment parsing."""
 
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
-        insert=[Ref("MsckRepairTableStatementSegment"), Ref("UnloadStatementSegment")],
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
+        insert=[
+            Ref("MsckRepairTableStatementSegment"),
+            Ref("UnloadStatementSegment"),
+            Ref("PrepareStatementSegment"),
+            Ref("ExecuteStatementSegment"),
+            Ref("ShowStatementSegment"),
+        ],
         remove=[
             Ref("TransactionStatementSegment"),
             Ref("CreateSchemaStatementSegment"),
@@ -336,7 +441,6 @@ class StatementSegment(ansi.StatementSegment):
             Ref("DropModelStatementSegment"),
         ],
     )
-    match_grammar = ansi.StatementSegment.match_grammar
 
 
 class CreateTableStatementSegment(BaseSegment):
@@ -379,7 +483,14 @@ class CreateTableStatementSegment(BaseSegment):
                     Bracketed(
                         Delimited(
                             Sequence(
-                                Ref("ColumnDefinitionSegment"),
+                                OneOf(
+                                    # External tables expect types...
+                                    Ref("ColumnDefinitionSegment"),
+                                    # Iceberg tables don't expect types.
+                                    Ref("SingleIdentifierGrammar"),
+                                    # Iceberg tables also allow partition transforms
+                                    Ref("FunctionSegment"),
+                                ),
                                 Ref("CommentGrammar", optional=True),
                             ),
                         ),
@@ -405,15 +516,17 @@ class CreateTableStatementSegment(BaseSegment):
             Sequence(
                 Sequence("WITH", Ref("BracketedCTASPropertyGrammar"), optional=True),
                 "AS",
-                OptionallyBracketed(Ref("SelectableGrammar")),
-                Sequence("WITH NO DATA", optional=True),
+                OptionallyBracketed(
+                    Ref("SelectableGrammar"),
+                ),
+                Sequence("WITH", "NO", "DATA", optional=True),
             ),
         ),
     )
 
 
 class MsckRepairTableStatementSegment(BaseSegment):
-    """An `MSCK REPAIR TABLE`statement.
+    """An `MSCK REPAIR TABLE` statement.
 
     The `MSCK REPAIR TABLE` command scans a file system such as Amazon S3 for
     Hive compatible partitions that were added to the file system after the
@@ -510,6 +623,49 @@ class UnloadStatementSegment(BaseSegment):
     )
 
 
+class PrepareStatementSegment(BaseSegment):
+    """A `prepare` statement.
+
+    https://docs.aws.amazon.com/athena/latest/ug/querying-with-prepared-statements.html
+    """
+
+    type = "prepare_statement"
+    match_grammar = Sequence(
+        "PREPARE",
+        Ref("TableReferenceSegment"),
+        "FROM",
+        OptionallyBracketed(
+            OneOf(
+                Ref("SelectableGrammar"),
+                Ref("UnloadStatementSegment"),
+                Ref("InsertStatementSegment"),
+            ),
+        ),
+    )
+
+
+class ExecuteStatementSegment(BaseSegment):
+    """An `execute` statement.
+
+    https://docs.aws.amazon.com/athena/latest/ug/querying-with-prepared-statements.html
+    """
+
+    type = "execute_statement"
+    match_grammar = Sequence(
+        "EXECUTE",
+        Ref("TableReferenceSegment"),
+        OneOf(
+            Sequence(
+                "USING",
+                Delimited(
+                    Ref("LiteralGrammar"),
+                ),
+            ),
+            optional=True,
+        ),
+    )
+
+
 class IntervalExpressionSegment(BaseSegment):
     """An interval expression segment.
 
@@ -532,3 +688,96 @@ class IntervalExpressionSegment(BaseSegment):
             ),
         ),
     )
+
+
+class GroupByClauseSegment(ansi.GroupByClauseSegment):
+    """A `GROUP BY` clause like in `SELECT`.
+
+    https://docs.aws.amazon.com/athena/latest/ug/select.html#:~:text=%5B-,GROUP,-BY%20%5B%20ALL%20%7C%20DISTINCT%20%5D%20grouping_expressions
+    """
+
+    match_grammar: Matchable = Sequence(
+        "GROUP",
+        "BY",
+        Indent,
+        Delimited(
+            OneOf(
+                Ref("CubeRollupClauseSegment"),
+                Ref("GroupingSetsClauseSegment"),
+                Ref("ColumnReferenceSegment"),
+                Ref("NumericLiteralSegment"),  # Can `GROUP BY 1`
+                Ref("ExpressionSegment"),  # Can `GROUP BY coalesce(col, 1)`
+            ),
+            terminators=[
+                Sequence("ORDER", "BY"),
+                "LIMIT",
+                "OFFSET",
+                "HAVING",
+                Ref("SetOperatorSegment"),
+            ],
+        ),
+        Dedent,
+    )
+
+
+class ShowStatementSegment(BaseSegment):
+    """A `show` execute statement.
+
+    Full Apache Hive `SHOW` reference:
+    https://cwiki.apache.org/confluence/display/Hive/LanguageManual+DDL#LanguageManualDDL-Show
+
+    Athena supported subset:
+    https://docs.aws.amazon.com/athena/latest/ug/ddl-reference.html
+    """
+
+    type = "show_statement"
+    match_grammar = Sequence(
+        "SHOW",
+        OneOf(
+            Sequence(
+                "COLUMNS",
+                OneOf("FROM", "IN"),
+                OneOf(
+                    Sequence(
+                        Ref("DatabaseReferenceSegment"), Ref("TableReferenceSegment")
+                    ),
+                    Sequence(
+                        Ref("TableReferenceSegment"),
+                        Sequence(
+                            OneOf("FROM", "IN"),
+                            Ref("DatabaseReferenceSegment"),
+                            optional=True,
+                        ),
+                    ),
+                ),
+            ),
+            Sequence(
+                "CREATE",
+                OneOf("TABLE", "VIEW"),
+                Ref("TableReferenceSegment"),
+            ),
+            Sequence(
+                OneOf("DATABASES", "SCHEMAS"),
+                Sequence("LIKE", Ref("QuotedLiteralSegment"), optional=True),
+            ),
+            Sequence(
+                "PARTITIONS",
+                Ref("TableReferenceSegment"),
+            ),
+            Sequence(
+                "TABLES",
+                Sequence("IN", Ref("DatabaseReferenceSegment"), optional=True),
+                Ref("QuotedLiteralSegment", optional=True),
+            ),
+            Sequence(
+                "TBLPROPERTIES",
+                Ref("TableReferenceSegment"),
+                Bracketed(Ref("QuotedLiteralSegment"), optional=True),
+            ),
+            Sequence(
+                "VIEWS",
+                Sequence("IN", Ref("DatabaseReferenceSegment"), optional=True),
+                Sequence("LIKE", Ref("QuotedLiteralSegment"), optional=True),
+            ),
+        ),
+    )
