@@ -523,6 +523,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("DeclareStatementSegment"),
             Ref("SetStatementSegment"),
             Ref("ExportStatementSegment"),
+            Ref("LoadDataStatementSegment"),
             Ref("CreateExternalTableStatementSegment"),
             Ref("CreateSnapshotTableStatementSegment"),
             Ref("ExecuteImmediateSegment"),
@@ -2534,7 +2535,7 @@ class MergeNotMatchedBySourceClauseSegment(ansi.MergeMatchedClauseSegment):
     """The `WHEN MATCHED BY SOURCE` clause within a `MERGE` statement.
 
     It inherits from `ansi.MergeMatchedClauseSegment` because NotMatchedBySource clause
-    is conceptionally more close to a Matched clause than to NotMatched clause, i.e.
+    is conceptually more close to a Matched clause than to NotMatched clause, i.e.
     it gets combined with an UPDATE or DELETE, not with an INSERT.
     """
 
@@ -2683,6 +2684,83 @@ class ExportStatementSegment(BaseSegment):
     )
 
 
+class LoadDataStatementSegment(BaseSegment):
+    """`LOAD DATA` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/load-statements
+    """
+
+    type = "load_data_statement"
+    match_grammar: Matchable = Sequence(
+        "LOAD",
+        "DATA",
+        OneOf("INTO", "OVERWRITE"),
+        Sequence(
+            Ref("TemporaryGrammar"),
+            "TABLE",
+            optional=True,
+        ),
+        Ref("TableReferenceSegment"),
+        Sequence(
+            Bracketed(
+                Delimited(
+                    Ref("ColumnDefinitionSegment"),
+                    Ref("TableConstraintSegment"),
+                    allow_trailing=True,
+                )
+            ),
+            optional=True,
+        ),
+        Sequence(
+            Sequence("OVERWRITE", optional=True),
+            "PARTITIONS",
+            Bracketed(
+                Delimited(
+                    Sequence(
+                        Ref("ParameterNameSegment"),
+                        Ref("EqualsSegment"),
+                        Ref("BaseExpressionElementGrammar"),
+                    )
+                )
+            ),
+            optional=True,
+        ),
+        Ref("PartitionBySegment", optional=True),
+        Ref("ClusterBySegment", optional=True),
+        Ref("OptionsSegment", optional=True),
+        Sequence(
+            "FROM",
+            "FILES",
+            Bracketed(
+                Delimited(
+                    Sequence(
+                        Ref("ParameterNameSegment"),
+                        Ref("EqualsSegment"),
+                        Ref("BaseExpressionElementGrammar"),
+                    )
+                )
+            ),
+        ),
+        Sequence(
+            "WITH",
+            "PARTITION",
+            "COLUMNS",
+            Bracketed(
+                Delimited(
+                    Sequence(
+                        Ref("SingleIdentifierGrammar"),  # Column name
+                        Ref("DatatypeSegment"),
+                    ),
+                    allow_trailing=True,
+                ),
+                optional=True,
+            ),
+            optional=True,
+        ),
+        Sequence("WITH", "CONNECTION", Ref("ObjectReferenceSegment"), optional=True),
+    )
+
+
 class ProcedureNameSegment(BaseSegment):
     """Procedure name, including any prefix bits, e.g. project or schema."""
 
