@@ -16,9 +16,9 @@ from sqlfluff.core.parser import (
     Nothing,
     OneOf,
     OptionallyBracketed,
-    ParseMode,
     Ref,
     RegexParser,
+    SegmentGenerator,
     Sequence,
     StringParser,
     SymbolSegment,
@@ -31,7 +31,11 @@ from sqlfluff.dialects.dialect_hive_keywords import (
 )
 
 ansi_dialect = load_raw_dialect("ansi")
-hive_dialect = ansi_dialect.copy_as("hive")
+hive_dialect = ansi_dialect.copy_as(
+    "hive",
+    formatted_name="Apache Hive",
+    docstring="The dialect for Apache `Hive <https://hive.apache.org/>`_.",
+)
 
 # Clear ANSI Keywords and add all Hive keywords
 # Commented clearing for now as some are needed for some statements imported
@@ -137,6 +141,7 @@ hive_dialect.add(
         "back_quote",
         IdentifierSegment,
         type="quoted_identifier",
+        casefold=str.lower,
     ),
 )
 
@@ -144,11 +149,28 @@ hive_dialect.add(
 hive_dialect.replace(
     JoinKeywordsGrammar=Sequence(Sequence("SEMI", optional=True), "JOIN"),
     QuotedLiteralSegment=OneOf(
-        TypedParser("single_quote", LiteralSegment, type="quoted_literal"),
-        TypedParser("double_quote", LiteralSegment, type="quoted_literal"),
-        TypedParser("back_quote", LiteralSegment, type="quoted_literal"),
+        TypedParser(
+            "single_quote", LiteralSegment, type="quoted_literal", casefold=str.lower
+        ),
+        TypedParser(
+            "double_quote", LiteralSegment, type="quoted_literal", casefold=str.lower
+        ),
+        TypedParser(
+            "back_quote", LiteralSegment, type="quoted_literal", casefold=str.lower
+        ),
     ),
     TrimParametersGrammar=Nothing(),
+    # ANSI with lower casefold
+    NakedIdentifierSegment=SegmentGenerator(
+        # Generate the anti template from the set of reserved keywords
+        lambda dialect: RegexParser(
+            r"[A-Z0-9_]*[A-Z][A-Z0-9_]*",
+            IdentifierSegment,
+            type="naked_identifier",
+            anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+            casefold=str.lower,
+        )
+    ),
     SingleIdentifierGrammar=ansi_dialect.get_grammar("SingleIdentifierGrammar").copy(
         insert=[
             Ref("BackQuotedIdentifierSegment"),
@@ -243,6 +265,19 @@ class ArrayTypeSegment(ansi.ArrayTypeSegment):
     )
 
 
+class EqualsSegment(ansi.EqualsSegment):
+    """Equals operator.
+
+    Hive allows double equals:
+    https://cwiki.apache.org/confluence/display/Hive/Hive+Operators
+    """
+
+    match_grammar: Matchable = Sequence(
+        Ref("RawEqualsSegment"),
+        Ref("RawEqualsSegment", optional=True),
+    )
+
+
 class StructTypeSegment(ansi.StructTypeSegment):
     """Expression to construct a STRUCT datatype."""
 
@@ -421,11 +456,32 @@ class TableConstraintSegment(ansi.TableConstraintSegment):
 class FromExpressionElementSegment(ansi.FromExpressionElementSegment):
     """Modified from ANSI to allow for `LATERAL VIEW` clause."""
 
-    match_grammar = ansi.FromExpressionElementSegment.match_grammar.copy(
-        insert=[
-            AnyNumberOf(Ref("LateralViewClauseSegment")),
-        ],
-        before=Ref("PostTableExpressionGrammar", optional=True),
+    match_grammar = (
+        ansi.FromExpressionElementSegment._base_from_expression_element.copy(
+            insert=[
+                AnyNumberOf(Ref("LateralViewClauseSegment")),
+            ],
+            before=Ref("PostTableExpressionGrammar", optional=True),
+        )
+    )
+
+
+class AliasExpressionSegment(ansi.AliasExpressionSegment):
+    """Modified to allow UDTF in SELECT clause to return multiple columns aliases.
+
+    Full Apache Hive `Built-in Table-Generating Functions (UDTF)` reference here:
+    https://cwiki.apache.org/confluence/display/hive/languagemanual+udf#LanguageManualUDF-Built-inTable-GeneratingFunctions(UDTF)
+    """
+
+    match_grammar = Sequence(
+        Ref.keyword("AS", optional=True),
+        OneOf(
+            Sequence(
+                Ref("SingleIdentifierGrammar", optional=True),
+                Bracketed(Ref("SingleIdentifierListSegment")),
+            ),
+            Ref("SingleIdentifierGrammar"),
+        ),
     )
 
 
@@ -663,6 +719,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("MsckRepairTableStatementSegment"),
             Ref("MsckTableStatementSegment"),
             Ref("SetStatementSegment"),
+            Ref("AlterViewStatementSegment"),
         ],
         remove=[
             Ref("TransactionStatementSegment"),
@@ -807,6 +864,22 @@ class MsckTableStatementSegment(BaseSegment):
     )
 
 
+class RowFunctionContentsSegment(BaseSegment):
+    """Row Function Contents."""
+
+    type = "function_contents"
+
+    match_grammar = Sequence(
+        Bracketed(
+            Delimited(
+                Sequence(
+                    Ref("BaseExpressionElementGrammar"),
+                ),
+            ),
+        ),
+    )
+
+
 class FunctionSegment(BaseSegment):
     """A scalar or aggregate function.
 
@@ -825,30 +898,14 @@ class FunctionSegment(BaseSegment):
             # rather than identifiers.
             Sequence(
                 Ref("DatePartFunctionNameSegment"),
-                Bracketed(
-                    Delimited(
-                        Ref("DatetimeUnitSegment"),
-                        Ref(
-                            "FunctionContentsGrammar",
-                            # The brackets might be empty for some functions...
-                            optional=True,
-                        ),
-                    ),
-                    parse_mode=ParseMode.GREEDY,
-                ),
+                Ref("DateTimeFunctionContentsSegment"),
             ),
         ),
         Sequence(
             # This unusual syntax is used to cast the Keyword ROW to
             # to the function_name to avoid rule linting exceptions
             StringParser("ROW", KeywordSegment, type="function_name"),
-            Bracketed(
-                Delimited(
-                    Sequence(
-                        Ref("BaseExpressionElementGrammar"),
-                    ),
-                ),
-            ),
+            Ref("RowFunctionContentsSegment"),
             "AS",
             "ROW",
             Bracketed(
@@ -869,14 +926,7 @@ class FunctionSegment(BaseSegment):
                         Ref("ValuesClauseSegment"),
                     ),
                 ),
-                Bracketed(
-                    Ref(
-                        "FunctionContentsGrammar",
-                        # The brackets might be empty for some functions...
-                        optional=True,
-                    ),
-                    parse_mode=ParseMode.GREEDY,
-                ),
+                Ref("FunctionContentsSegment"),
             ),
             Ref("PostFunctionGrammar", optional=True),
         ),
@@ -1050,3 +1100,25 @@ class SortByClauseSegment(ansi.OrderByClauseSegment):
         ),
         Dedent,
     )
+
+
+class AlterViewStatementSegment(BaseSegment):
+    """A `ALTER VIEW` statement to change the view schema or properties.
+
+    https://cwiki.apache.org/confluence/display/Hive/LanguageManual+DDL#LanguageManualDDL-Create/Drop/AlterView
+    """
+
+    type = "alter_view_statement"
+
+    match_grammar = Sequence(
+        "ALTER",
+        "VIEW",
+        Ref("TableReferenceSegment"),
+        OneOf(
+            Sequence("SET", Ref("TablePropertiesGrammar")),
+            Sequence(
+                "AS",
+                OptionallyBracketed(Ref("SelectStatementSegment")),
+            ),
+        ),
+    )
