@@ -2,6 +2,7 @@
 
 This inherits from the ansi dialect.
 """
+
 from typing import cast
 
 from sqlfluff.core.dialects import load_raw_dialect
@@ -14,10 +15,12 @@ from sqlfluff.core.parser import (
     BracketedSegment,
     CodeSegment,
     CommentSegment,
+    CompositeComparisonOperatorSegment,
     Delimited,
     IdentifierSegment,
     LiteralSegment,
     Matchable,
+    Nothing,
     OneOf,
     OptionallyBracketed,
     ParseMode,
@@ -35,7 +38,13 @@ from sqlfluff.core.parser import (
 from sqlfluff.dialects import dialect_ansi as ansi
 
 ansi_dialect = load_raw_dialect("ansi")
-oracle_dialect = ansi_dialect.copy_as("oracle")
+oracle_dialect = ansi_dialect.copy_as(
+    "oracle",
+    formatted_name="Oracle",
+    docstring="""The dialect for `Oracle`_ SQL. Note: this does not include PL/SQL.
+
+.. _`Oracle`: https://www.oracle.com/database/technologies/appdev/sql.html""",
+)
 
 oracle_dialect.sets("unreserved_keywords").difference_update(["COMMENT"])
 oracle_dialect.sets("reserved_keywords").update(
@@ -61,7 +70,7 @@ oracle_dialect.sets("reserved_keywords").update(
 )
 
 oracle_dialect.sets("unreserved_keywords").update(
-    ["EDITIONABLE", "EDITIONING", "NONEDITIONABLE", "KEEP"]
+    ["EDITIONABLE", "EDITIONING", "NONEDITIONABLE", "KEEP", "NOMINVALUE", "NOMAXVALUE"]
 )
 
 oracle_dialect.sets("bare_functions").clear()
@@ -85,6 +94,19 @@ oracle_dialect.patch_lexer_matchers(
             "single_quote",
             r"'([^'\\]|\\|\\.|'')*'",
             CodeSegment,
+            segment_kwargs={
+                "quoted_value": (r"'((?:[^'\\]|\\|\\.|'')*)'", 1),
+                "escape_replacements": [(r"''", "'")],
+            },
+        ),
+        RegexLexer(
+            "double_quote",
+            r'"([^"]|"")*"',
+            CodeSegment,
+            segment_kwargs={
+                "quoted_value": (r'"((?:[^"]|"")*)"', 1),
+                "escape_replacements": [(r'""', '"')],
+            },
         ),
     ]
 )
@@ -172,6 +194,7 @@ oracle_dialect.replace(
             IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+            casefold=str.upper,
         )
     ),
     PostFunctionGrammar=AnyNumberOf(
@@ -207,6 +230,7 @@ oracle_dialect.replace(
     ).copy(
         insert=[
             Ref("ConnectByRootGrammar"),
+            Ref("SqlplusSubstitutionVariableSegment"),
         ]
     ),
     Expression_D_Grammar=Sequence(
@@ -266,6 +290,7 @@ oracle_dialect.replace(
                 ),
             ),
             Ref("LocalAliasSegment"),
+            Ref("SqlplusSubstitutionVariableSegment"),
             terminators=[Ref("CommaSegment")],
         ),
         Ref("AccessorGrammar", optional=True),
@@ -279,6 +304,22 @@ oracle_dialect.replace(
             Sequence("TO", Ref("IntervalUnitsGrammar"), optional=True),
         ),
     ),
+    PreTableFunctionKeywordsGrammar=OneOf("LATERAL"),
+    ConditionalCrossJoinKeywordsGrammar=Nothing(),
+    UnconditionalCrossJoinKeywordsGrammar=Ref.keyword("CROSS"),
+    SingleIdentifierGrammar=ansi_dialect.get_grammar("SingleIdentifierGrammar").copy(
+        insert=[
+            Ref("SqlplusSubstitutionVariableSegment"),
+        ]
+    ),
+    SequenceMinValueGrammar=OneOf(
+        Sequence("MINVALUE", Ref("NumericLiteralSegment")),
+        "NOMINVALUE",
+    ),
+    SequenceMaxValueGrammar=OneOf(
+        Sequence("MAXVALUE", Ref("NumericLiteralSegment")),
+        "NOMAXVALUE",
+    ),
 )
 
 
@@ -376,7 +417,7 @@ class AlterTableColumnClausesSegment(BaseSegment):
             Ref("ColumnReferenceSegment"),
             "TO",
             Ref("ColumnReferenceSegment"),
-        )
+        ),
         # @TODO: modify_collection_retrieval
         # @TODO: modify_LOB_storage_clause
         # @TODO: alter_varray_col_properties
@@ -589,6 +630,7 @@ class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
             "NONEDITIONABLE",
             optional=True,
         ),
+        Ref.keyword("MATERIALIZED", optional=True),
         "VIEW",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
@@ -702,7 +744,18 @@ class ColumnDefinitionSegment(BaseSegment):
             ),
             Sequence(
                 Ref("DatatypeSegment"),  # Column type
-                Bracketed(Anything(), optional=True),  # For types like VARCHAR(100)
+                # For types like VARCHAR(100), VARCHAR(100 BYTE), VARCHAR (100 CHAR)
+                Bracketed(
+                    Sequence(
+                        Anything(),
+                        OneOf(
+                            "BYTE",
+                            "CHAR",
+                            optional=True,
+                        ),
+                    ),
+                    optional=True,
+                ),
                 AnyNumberOf(
                     Ref("ColumnConstraintSegment", optional=True),
                 ),
@@ -759,7 +812,7 @@ class StartWithClauseSegment(BaseSegment):
 
 
 class HierarchicalQueryClauseSegment(BaseSegment):
-    """Hiearchical Query.
+    """Hierarchical Query.
 
     https://docs.oracle.com/en/database/oracle/oracle-database/21/sqlrf/Hierarchical-Queries.html
     """
@@ -826,7 +879,7 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
     )
 
 
-class GreaterThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
+class GreaterThanOrEqualToSegment(CompositeComparisonOperatorSegment):
     """Allow spaces between operators."""
 
     match_grammar = OneOf(
@@ -841,7 +894,7 @@ class GreaterThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
     )
 
 
-class LessThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
+class LessThanOrEqualToSegment(CompositeComparisonOperatorSegment):
     """Allow spaces between operators."""
 
     match_grammar = OneOf(
@@ -856,7 +909,7 @@ class LessThanOrEqualToSegment(ansi.CompositeComparisonOperatorSegment):
     )
 
 
-class NotEqualToSegment(ansi.CompositeComparisonOperatorSegment):
+class NotEqualToSegment(CompositeComparisonOperatorSegment):
     """Allow spaces between operators."""
 
     match_grammar = OneOf(
@@ -945,3 +998,28 @@ class FunctionNameSegment(BaseSegment):
         ),
         allow_gaps=False,
     )
+
+
+class SqlplusSubstitutionVariableSegment(BaseSegment):
+    """SQLPlus Substitution Variables &thing.
+
+    https://docs.oracle.com/en/database/oracle/oracle-database/21/sqpug/using-substitution-variables-sqlplus.html
+    """
+
+    type = "sqlplus_variable"
+
+    match_grammar = Sequence(
+        Ref("AmpersandSegment"),
+        Ref("AmpersandSegment", optional=True),
+        Ref("SingleIdentifierGrammar"),
+    )
+
+
+class TableExpressionSegment(ansi.TableExpressionSegment):
+    """The main table expression e.g. within a FROM clause."""
+
+    match_grammar = ansi.TableExpressionSegment.match_grammar.copy(
+        insert=[
+            Ref("SqlplusSubstitutionVariableSegment"),
+        ]
+    )
