@@ -168,6 +168,7 @@ postgres_docs_keywords = [
     ("COMMENTS", "non-reserved"),
     ("COMMIT", "non-reserved"),
     ("COMMITTED", "non-reserved"),
+    ("COMMUTATOR", "reserved-(can-be-function-or-type)"),
     ("COMPRESSION", "non-reserved"),
     ("CONCURRENTLY", "reserved-(can-be-function-or-type)"),
     ("CONDITION", "not-keyword"),
@@ -357,6 +358,7 @@ postgres_docs_keywords = [
     ("GROUPING", "non-reserved-(cannot-be-function-or-type)"),
     ("GROUPS", "non-reserved"),
     ("HANDLER", "non-reserved"),
+    ("HASHES", "reserved-(can-be-function-or-type)"),
     ("HAVING", "reserved"),
     ("HEADER", "non-reserved"),
     ("HEX", "not-keyword"),
@@ -445,6 +447,7 @@ postgres_docs_keywords = [
     ("LEAKPROOF", "non-reserved"),
     ("LEAST", "non-reserved-(cannot-be-function-or-type)"),
     ("LEFT", "reserved-(can-be-function-or-type)"),
+    ("LEFTARG", "non-reserved-(cannot-be-function-or-type)"),
     ("LENGTH", "not-keyword"),
     ("LEVEL", "non-reserved"),
     ("LIBRARY", "not-keyword"),
@@ -485,6 +488,7 @@ postgres_docs_keywords = [
     ("MEASURES", "not-keyword"),
     ("MEMBER", "not-keyword"),
     ("MERGE", "non-reserved"),
+    ("MERGES", "reserved-(can-be-function-or-type)"),
     ("MESSAGE_LENGTH", "not-keyword"),
     ("MESSAGE_OCTET_LENGTH", "not-keyword"),
     ("MESSAGE_TEXT", "not-keyword"),
@@ -509,6 +513,7 @@ postgres_docs_keywords = [
     ("NATURAL", "reserved-(can-be-function-or-type)"),
     ("NCHAR", "non-reserved-(cannot-be-function-or-type)"),
     ("NCLOB", "not-keyword"),
+    ("NEGATOR", "reserved-(can-be-function-or-type)"),
     ("NESTED", "not-keyword"),
     ("NESTING", "not-keyword"),
     ("NEW", "non-reserved"),
@@ -677,6 +682,7 @@ postgres_docs_keywords = [
     ("RETURNS", "non-reserved"),
     ("REVOKE", "non-reserved"),
     ("RIGHT", "reserved-(can-be-function-or-type)"),
+    ("RIGHTARG", "reserved-(can-be-function-or-type)"),
     ("ROLE", "non-reserved"),
     ("ROLLBACK", "non-reserved"),
     ("ROLLUP", "non-reserved"),
