@@ -3,6 +3,7 @@
 This is based on postgres dialect, since it was initially based off of Postgres 8.
 We should monitor in future and see if it should be rebased off of ANSI
 """
+
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
@@ -13,12 +14,11 @@ from sqlfluff.core.parser import (
     Dedent,
     Delimited,
     IdentifierSegment,
-    Indent,
+    ImplicitIndent,
     Matchable,
     Nothing,
     OneOf,
     OptionallyBracketed,
-    ParseMode,
     Ref,
     RegexLexer,
     RegexParser,
@@ -35,7 +35,22 @@ from sqlfluff.dialects.dialect_redshift_keywords import (
 
 postgres_dialect = load_raw_dialect("postgres")
 ansi_dialect = load_raw_dialect("ansi")
-redshift_dialect = postgres_dialect.copy_as("redshift")
+redshift_dialect = postgres_dialect.copy_as(
+    "redshift",
+    formatted_name="AWS Redshift",
+    docstring="""**Default Casing**: ``lowercase`` (unless configured
+to be case sensitive with all identifiers using the
+:code:`enable_case_sensitive_identifier` configuration value, see
+the `Redshift Names & Identifiers Docs`_).
+
+**Quotes**: String Literals: ``''``, Identifiers: ``""``.
+
+The dialect for `Redshift`_ on Amazon Web Services (AWS).
+
+.. _`Redshift`: https://aws.amazon.com/redshift/
+.. _`Redshift Names & Identifiers Docs`: https://spark.apache.org/docs/latest/sql-ref.html
+""",  # noqa: E501
+)
 
 # Set Keywords
 redshift_dialect.sets("unreserved_keywords").clear()
@@ -189,6 +204,7 @@ redshift_dialect.replace(
             IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+            casefold=str.lower,
         )
     ),
 )
@@ -322,10 +338,7 @@ class DateTimeTypeIdentifier(BaseSegment):
     match_grammar = OneOf(
         "DATE",
         "DATETIME",
-        Sequence(
-            OneOf("TIME", "TIMESTAMP"),
-            Sequence(OneOf("WITH", "WITHOUT"), "TIME", "ZONE", optional=True),
-        ),
+        Ref("TimeWithTZGrammar"),
         OneOf("TIMETZ", "TIMESTAMPTZ"),
         # INTERVAL types are not Datetime types under Redshift:
         # https://docs.aws.amazon.com/redshift/latest/dg/r_Datetime_types.html
@@ -686,6 +699,13 @@ class AlterTableActionSegment(BaseSegment):
             Ref("ColumnReferenceSegment"),
             Ref("DropBehaviorGrammar", optional=True),
         ),
+        Sequence(
+            "APPEND",
+            "FROM",
+            Ref("TableReferenceSegment"),
+            Ref.keyword("IGNOREEXTRA", optional=True),
+            Ref.keyword("FILLTARGET", optional=True),
+        ),
     )
 
 
@@ -740,7 +760,7 @@ class TableConstraintSegment(BaseSegment):
                 Bracketed(Delimited(Ref("ColumnReferenceSegment"))),
                 "REFERENCES",
                 Ref("TableReferenceSegment"),
-                Sequence(Bracketed(Ref("ColumnReferenceSegment"))),
+                Bracketed(Delimited(Ref("ColumnReferenceSegment"))),
             ),
         ),
     )
@@ -1250,6 +1270,21 @@ class UnloadStatementSegment(BaseSegment):
                 ),
                 optional=True,
             ),
+            Sequence(
+                "EXTENSION",
+                Ref("QuotedLiteralSegment"),
+                Sequence(
+                    "PARALLEL",
+                    OneOf(
+                        "ON",
+                        "OFF",
+                        "TRUE",
+                        "FALSE",
+                    ),
+                    optional=True,
+                ),
+                optional=True,
+            ),
             OneOf(
                 Sequence(
                     "DELIMITER",
@@ -1835,7 +1870,7 @@ class GrantUsageDatashareStatementSegment(BaseSegment):
 
     https://docs.aws.amazon.com/redshift/latest/dg/r_GRANT.html
     section "Granting datashare permissions"
-    Note: According to docummentation, multiple accounts and namespaces can be
+    Note: According to documentation, multiple accounts and namespaces can be
           specified. However, tests using redshift instance showed this causes a syntax
           error.
     """
@@ -2046,6 +2081,9 @@ class StatementSegment(postgres.StatementSegment):
             Ref("CreateExternalFunctionStatementSegment"),
             Ref("GrantUsageDatashareStatementSegment"),
         ],
+        remove=[
+            Ref("ShowStatementSegment"),
+        ],
     )
 
 
@@ -2473,17 +2511,7 @@ class FunctionSegment(ansi.FunctionSegment):
             # rather than identifiers.
             Sequence(
                 Ref("DatePartFunctionNameSegment"),
-                Bracketed(
-                    Delimited(
-                        Ref("DatetimeUnitSegment"),
-                        Ref(
-                            "FunctionContentsGrammar",
-                            # The brackets might be empty for some functions...
-                            optional=True,
-                        ),
-                    ),
-                    parse_mode=ParseMode.GREEDY,
-                ),
+                Ref("DateTimeFunctionContentsSegment"),
             ),
         ),
         Sequence(
@@ -2509,24 +2537,27 @@ class FunctionSegment(ansi.FunctionSegment):
                         ),
                     ),
                 ),
-                Bracketed(
-                    Ref(
-                        "FunctionContentsGrammar",
-                        # The brackets might be empty for some functions...
-                        optional=True,
-                    ),
-                    parse_mode=ParseMode.GREEDY,
-                ),
+                Ref("FunctionContentsSegment"),
             ),
             Ref("PostFunctionGrammar", optional=True),
         ),
         Sequence(
             Ref("ConvertFunctionNameSegment"),
-            Bracketed(
-                Ref("DatatypeSegment"),
-                Ref("CommaSegment"),
-                Ref("ExpressionSegment"),
-            ),
+            Ref("ConvertFunctionContentsSegment"),
+        ),
+    )
+
+
+class ConvertFunctionContentsSegment(BaseSegment):
+    """Convert Function contents."""
+
+    type = "function_contents"
+
+    match_grammar = Sequence(
+        Bracketed(
+            Ref("DatatypeSegment"),
+            Ref("CommaSegment"),
+            Ref("ExpressionSegment"),
         ),
     )
 
@@ -2634,7 +2665,7 @@ class QualifyClauseSegment(BaseSegment):
     type = "qualify_clause"
     match_grammar = Sequence(
         "QUALIFY",
-        Indent,
+        ImplicitIndent,
         Ref("ExpressionSegment"),
         Dedent,
     )
