@@ -103,6 +103,7 @@ ADD
 ADMIN
 AFTER
 AGGREGATE
+ALERT
 ALLOW_DUPLICATE
 ALLOW_OVERLAPPING_EXECUTION
 ALLOW_WRITES
@@ -113,6 +114,7 @@ ALWAYS
 API
 API_INTEGRATION
 APPEND_ONLY
+APPLICATION
 APPLY
 ARRAY
 ASC
@@ -156,6 +158,7 @@ BLOCK
 BLOCKED_IP_LIST
 BLOCKED_NETWORK_RULE_LIST
 BODY
+BOOLEAN
 BROTLI
 BZ2
 CACHE
@@ -249,6 +252,7 @@ EMPTY
 EMPTY_FIELD_AS_NULL
 ENABLE
 ENABLE_OCTAL
+ENABLE_QUERY_ACCELERATION
 ENABLED
 ENCODING
 ENCRYPTION
@@ -484,6 +488,7 @@ PURGE
 PUT
 PYTHON
 QUERIES
+QUERY_ACCELERATION_MAX_SCALE_FACTOR
 QUERY_WAREHOUSE
 QUEUE
 RANGE
