@@ -11,6 +11,7 @@ Teradata Database SQL Data Definition Language Syntax and Examples
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
+    AnySetOf,
     Anything,
     BaseSegment,
     Bracketed,
@@ -19,6 +20,8 @@ from sqlfluff.core.parser import (
     CompositeComparisonOperatorSegment,
     Dedent,
     Delimited,
+    IdentifierSegment,
+    ImplicitIndent,
     Indent,
     Matchable,
     OneOf,
@@ -27,11 +30,16 @@ from sqlfluff.core.parser import (
     RegexLexer,
     Sequence,
     StringParser,
+    TypedParser,
 )
 from sqlfluff.dialects import dialect_ansi as ansi
 
 ansi_dialect = load_raw_dialect("ansi")
-teradata_dialect = ansi_dialect.copy_as("teradata")
+teradata_dialect = ansi_dialect.copy_as(
+    "teradata",
+    formatted_name="Teradata",
+    docstring="""The dialect for `Teradata <https://www.teradata.co.uk/>`_.""",
+)
 
 teradata_dialect.patch_lexer_matchers(
     [
@@ -64,6 +72,7 @@ teradata_dialect.sets("unreserved_keywords").update(
         "DEL",
         "DUAL",
         "ERRORCODE",
+        "EXCL",
         "EXPORT",
         "FALLBACK",
         "FORMAT",
@@ -79,6 +88,7 @@ teradata_dialect.sets("unreserved_keywords").update(
         "MEETS",
         "MERGEBLOCKRATIO",
         "NONE",
+        "OVERRIDE",
         "PERCENT",
         "PROFILE",
         "PROTECTION",
@@ -98,7 +108,9 @@ teradata_dialect.sets("unreserved_keywords").update(
     ]
 )
 
-teradata_dialect.sets("reserved_keywords").update(["UNION", "TIMESTAMP"])
+teradata_dialect.sets("reserved_keywords").update(
+    ["LOCKING", "UNION", "REPLACE", "TIMESTAMP"]
+)
 
 teradata_dialect.sets("bare_functions").update(["DATE"])
 
@@ -122,7 +134,11 @@ teradata_dialect.replace(
         Ref("LikeOperatorSegment"),
         Sequence("IS", "DISTINCT", "FROM"),
         Sequence("IS", "NOT", "DISTINCT", "FROM"),
-    )
+    ),
+    # match ANSI's naked identifier casefold, teradata is case-insensitive.
+    QuotedIdentifierSegment=TypedParser(
+        "double_quote", IdentifierSegment, type="quoted_identifier", casefold=str.upper
+    ),
 )
 
 teradata_dialect.add(
@@ -633,12 +649,14 @@ class CreateTableStatementSegment(BaseSegment):
     type = "create_table_statement"
     match_grammar = Sequence(
         "CREATE",
-        Sequence("OR", "REPLACE", optional=True),
-        # Adding Teradata specific [MULTISET| SET]
-        OneOf("SET", "MULTISET", optional=True),
-        OneOf(Sequence("GLOBAL", "TEMPORARY"), "VOLATILE", optional=True),
+        Ref("OrReplaceGrammar", optional=True),
+        AnySetOf(
+            OneOf("SET", "MULTISET"),
+            OneOf(Sequence("GLOBAL", "TEMPORARY"), "VOLATILE"),
+            optional=True,
+        ),
         "TABLE",
-        Sequence("IF", "NOT", "EXISTS", optional=True),
+        Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
         # , NO FALLBACK, NO BEFORE JOURNAL, NO AFTER JOURNAL
         Ref("TdCreateTableOptions", optional=True),
@@ -665,6 +683,23 @@ class CreateTableStatementSegment(BaseSegment):
     )
 
 
+class CreateViewStatementSegment(BaseSegment):
+    """A `[CREATE | REPLACE] VIEW` statement."""
+
+    type = "create_view_statement"
+    match_grammar: Matchable = Sequence(
+        OneOf("CREATE", "REPLACE"),
+        "VIEW",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        # Optional list of column names
+        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        "AS",
+        OptionallyBracketed(Ref("SelectableGrammar")),
+        Ref("WithNoSchemaBindingClauseSegment", optional=True),
+    )
+
+
 # Update
 class UpdateStatementSegment(BaseSegment):
     """A `Update from` statement.
@@ -679,6 +714,7 @@ class UpdateStatementSegment(BaseSegment):
     type = "update_statement"
     match_grammar = Sequence(
         "UPDATE",
+        Indent,
         OneOf(
             Ref("TableReferenceSegment"),
             Ref("FromUpdateClauseSegment"),
@@ -687,6 +723,7 @@ class UpdateStatementSegment(BaseSegment):
                 Ref("FromUpdateClauseSegment"),
             ),
         ),
+        Dedent,
         Ref("SetClauseListSegment"),
         Ref("WhereClauseSegment", optional=True),
     )
@@ -731,23 +768,76 @@ class QualifyClauseSegment(BaseSegment):
     type = "qualify_clause"
     match_grammar = Sequence(
         "QUALIFY",
-        Indent,
+        ImplicitIndent,
         OptionallyBracketed(Ref("ExpressionSegment")),
         Dedent,
     )
 
 
+class LockingClauseSegment(BaseSegment):
+    """A `LOCKING` clause for Teradata.
+
+    https://docs.teradata.com/r/Enterprise_IntelliFlex_VMware/SQL-Data-Manipulation-Language/Statement-Syntax/LOCKING-Request-Modifier
+    """
+
+    type = "locking_clause"
+    match_grammar = Sequence(
+        OneOf(
+            "LOCKING",
+            "LOCK",
+        ),
+        OneOf(
+            "ROW",
+            Sequence("TABLE", Ref("ObjectReferenceSegment", optional=True)),
+            Sequence("VIEW", Ref("ObjectReferenceSegment", optional=True)),
+            Sequence("DATABASE", Ref("ObjectReferenceSegment", optional=True)),
+        ),
+        OneOf(
+            "FOR",
+            "IN",
+        ),
+        OneOf(
+            "ACCESS",
+            "WRITE",
+            "EXCLUSIVE",
+            "EXCL",
+            Sequence("READ", Sequence("OVERRIDE", optional=True)),
+            "SHARE",
+            "CHECKSUM",
+            Sequence("LOAD", "COMMITTED"),
+        ),
+        Sequence("MODE", optional=True),
+        Sequence("NOWAIT", optional=True),
+    )
+
+
 class SelectStatementSegment(ansi.SelectStatementSegment):
     """A `SELECT` statement.
 
     https://dev.mysql.com/doc/refman/5.7/en/select.html
     """
 
-    match_grammar = ansi.SelectStatementSegment.match_grammar.copy(
+    match_grammar_with_qualify_clause = ansi.SelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
     )
 
+    match_grammar = match_grammar_with_qualify_clause.copy(
+        insert=[Ref("LockingClauseSegment", optional=True)],
+        before=Ref("SelectClauseSegment"),
+    )
+
+
+class WithCompoundStatementSegment(ansi.WithCompoundStatementSegment):
+    """A `SELECT` statement preceded by a selection of `WITH` clauses.
+
+    `WITH tab (col1,col2) AS (SELECT a,b FROM x)`
+    """
+
+    match_grammar = ansi.WithCompoundStatementSegment.match_grammar.copy(
+        insert=[Ref("LockingClauseSegment", optional=True)], at=0
+    )
+
 
 class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     """An unordered `SELECT` statement.
