@@ -46,9 +46,9 @@ RESERVED_KEYWORDS = [
     "BY",
     "CASCADE",
     "CASE",
+    "CHECK_CONSTRAINTS",
     "CHECK",
     "CHECKPOINT",
-    "CHECK_CONSTRAINTS",
     "CLOSE",
     "CLUSTERED",
     "COALESCE",
@@ -63,7 +63,6 @@ RESERVED_KEYWORDS = [
     "CONVERT",
     "CREATE",
     "CROSS",
-    "CURRENT",
     "CURRENT_CATALOG",  # *future*
     "CURRENT_DATE",
     "CURRENT_DEFAULT_TRANSFORM_GROUP",  # *future*
@@ -74,6 +73,7 @@ RESERVED_KEYWORDS = [
     "CURRENT_TIMESTAMP",
     "CURRENT_TRANSFORM_GROUP_FOR_TYPE",  # *future*
     "CURRENT_USER",
+    "CURRENT",
     "CURSOR",
     "DATABASE",
     "DBCC",
@@ -103,8 +103,8 @@ RESERVED_KEYWORDS = [
     "FILE",
     "FILLFACTOR",
     "FOR",
-    "FORWARD_ONLY",
     "FOREIGN",
+    "FORWARD_ONLY",
     "FREETEXT",
     "FREETEXTTABLE",
     "FROM",
@@ -169,11 +169,10 @@ RESERVED_KEYWORDS = [
     "PRINT",
     "PROC",
     "PROCEDURE",
-    "PROPERTY",
     "PUBLIC",
     "RAISERROR",
-    "READ",
     "READ_ONLY",
+    "READ",
     "READTEXT",
     "RECONFIGURE",
     "REFERENCES",
@@ -191,8 +190,8 @@ RESERVED_KEYWORDS = [
     "RULE",
     "SAVE",
     "SCHEMA",
-    "SCROLL",
     "SCROLL_LOCKS",
+    "SCROLL",
     "SELECT",
     "SEMANTICKEYPHRASETABLE",
     "SEMANTICSIMILARITYDETAILSTABLE",
@@ -212,8 +211,8 @@ RESERVED_KEYWORDS = [
     "TO",
     "TOP",
     "TRAN",
-    "TRANSACTION",
     "TRAN",
+    "TRANSACTION",
     "TRIGGER",
     "TRUNCATE",
     "TRY_CONVERT",
@@ -249,15 +248,16 @@ FUTURE_RESERVED_KEYWORDS = [
 ]
 
 UNRESERVED_KEYWORDS = [
-    "ABORT",
     "ABORT_AFTER_WAIT",
+    "ABORT",
     "ABSENT",
-    "ATOMIC",
+    "ACTION",
     "AFTER",
     "ALGORITHM",
-    "ALLOWED",
+    "ALLOW_ENCRYPTED_VALUE_MODIFICATIONS",
     "ALLOW_PAGE_LOCKS",
     "ALLOW_ROW_LOCKS",
+    "ALLOWED",
     "ALWAYS",
     "ANSI_DEFAULTS",
     "ANSI_NULL_DFLT_OFF",
@@ -269,7 +269,9 @@ UNRESERVED_KEYWORDS = [
     "APPLY",
     "ARITHABORT",
     "ARITHIGNORE",
+    "ASYMMETRIC",
     "AT",
+    "ATOMIC",
     "AUTO_CREATE_TABLE",
     "AUTO",
     "BEFORE",  # *future*
@@ -282,7 +284,10 @@ UNRESERVED_KEYWORDS = [
     "CALLER",
     "CAST",
     "CATCH",
+    "CERTIFICATE",
     "CHANGE_TRACKING",
+    "CHECK_EXPIRATION",
+    "CHECK_POLICY",
     "CODEPAGE",
     "COLUMN_ENCRYPTION_KEY",
     "COLUMNSTORE_ARCHIVE",
@@ -295,10 +300,11 @@ UNRESERVED_KEYWORDS = [
     "CONCAT",
     "CONNECTION_OPTIONS",
     "CONTAINED",
+    "CONTEXT_INFO",
     "CONTINUE",
     "CONTROL",
-    "CREDENTIAL",
     "COPY",
+    "CREDENTIAL",
     "CURSOR_CLOSE_ON_COMMIT",
     "CYCLE",
     "DATA_COMPRESSION",
@@ -315,6 +321,9 @@ UNRESERVED_KEYWORDS = [
     "DAY",
     "DAYS",
     "DEADLOCK_PRIORITY",
+    "DEFAULT_DATABASE",
+    "DEFAULT_LANGUAGE",
+    "DEFAULT_SCHEMA",
     "DELAY",
     "DELAYED_DURABILITY",
     "DELIMITEDTEXT",
@@ -345,11 +354,11 @@ UNRESERVED_KEYWORDS = [
     "FIELDQUOTE",
     "FIELDTERMINATOR",
     "FILE_FORMAT",
+    "FILE_TYPE",
     "FILEGROUP",
-    "FILESTREAM",
     "FILESTREAM_ON",
     "FILESTREAM",
-    "FILE_TYPE",
+    "FILESTREAM",
     "FILETABLE_COLLATE_FILENAME",
     "FILETABLE_DIRECTORY",
     "FILETABLE_FULLPATH_UNIQUE_CONSTRAINT_NAME",
@@ -396,6 +405,7 @@ UNRESERVED_KEYWORDS = [
     "INCREMENT",
     "INFINITE",
     "INLINE",
+    "INPUT",
     "INSTEAD",
     "INTERVAL",
     "IO",
@@ -411,8 +421,8 @@ UNRESERVED_KEYWORDS = [
     "LANGUAGE",
     "LAST",
     "LASTROW",
-    "LEDGER",
     "LEDGER_VIEW",
+    "LEDGER",
     "LEGACY_CARDINALITY_ESTIMATION",
     "LEVEL",
     "LOAD",  # listed as reserved but functionally unreserved
@@ -425,6 +435,7 @@ UNRESERVED_KEYWORDS = [
     "LOW",
     "MANUAL",
     "MASKED",
+    "MASTER",
     "MATCHED",
     "MAX_DURATION",
     "MAX_GRANT_PERCENT",
@@ -440,6 +451,7 @@ UNRESERVED_KEYWORDS = [
     "MINVALUE",
     "MONTH",
     "MONTHS",
+    "MUST_CHANGE",
     "NAME",
     "NEXT",
     "NO_PERFORMANCE_SPOOL",
@@ -452,10 +464,12 @@ UNRESERVED_KEYWORDS = [
     "NORMAL",
     "NOWAIT",
     "NTILE",
+    "NULLS",
     "NUMERIC_ROUNDABORT",
     "OBJECT",
     "OFFSET",
     "ONLINE",
+    "ONLY",
     "OPENJSON",
     "OPERATION_TYPE_COLUMN_NAME",
     "OPERATION_TYPE_DESC_COLUMN_NAME",
@@ -465,19 +479,21 @@ UNRESERVED_KEYWORDS = [
     "OUT",
     "OUTBOUND",
     "OUTPUT",
+    "OVERRIDE",
     "OWNER",
     "PAD_INDEX",
     "PAGE",
     "PAGLOCK",
     "PARAMETER",
-    "PARAMETERS",  # *future*
     "PARAMETERIZATION",
+    "PARAMETERS",  # *future*
     "PARQUET",
     "PARSEONLY",
     "PARSER_VERSION",
     "PARTIAL",  # *future*
     "PARTITION",
     "PARTITIONS",
+    "PASSWORD",
     "PATH",
     "PAUSE",
     "PAUSED",
@@ -491,6 +507,8 @@ UNRESERVED_KEYWORDS = [
     "PRECISION",  # listed as reserved but functionally unreserved
     "PRIOR",
     "PROFILE",
+    "PROPERTY",
+    "PROVIDER",
     "PUSHDOWN",
     "QUERY_GOVERNOR_COST_LIMIT",
     "QUERYTRACEON",
@@ -510,6 +528,7 @@ UNRESERVED_KEYWORDS = [
     "RECEIVE",
     "RECOMPILE",
     "RECURSIVE",
+    "REGENERATE",
     "REGR_AVGX",  # *future*
     "REGR_AVGY",  # *future*
     "REGR_COUNT",  # *future*
@@ -519,10 +538,10 @@ UNRESERVED_KEYWORDS = [
     "REGR_SXX",  # *future*
     "REGR_SXY",  # *future*
     "REGR_SYY",  # *future*
-    "REJECTED_ROW_LOCATION",
     "REJECT_SAMPLE_VALUE",
     "REJECT_TYPE",
     "REJECT_VALUE",
+    "REJECTED_ROW_LOCATION",
     "REMOTE_DATA_ARCHIVE",
     "REMOTE_PROC_TRANSACTIONS",
     "RENAME",  # Azure Synapse Analytics specific
@@ -539,6 +558,7 @@ UNRESERVED_KEYWORDS = [
     "RETURNS",
     "ROBUST",
     "ROLE",
+    "ROLLUP",
     "ROOT",
     "ROUND_ROBIN",  # Azure Synapse Analytics specific
     "ROW_NUMBER",
@@ -553,6 +573,7 @@ UNRESERVED_KEYWORDS = [
     "SCHEMA_AND_DATA",
     "SCHEMA_ONLY",
     "SCHEMABINDING",
+    "SCHEME",
     "SCOPED",
     "SEARCH",
     "SECRET",
@@ -564,11 +585,13 @@ UNRESERVED_KEYWORDS = [
     "SERDE_METHOD",
     "SERIALIZABLE",
     "SERVER",
+    "SERVICE",
     "SETERROR",
     "SETVAR",  # sqlcmd command
     "SHOWPLAN_ALL",
     "SHOWPLAN_TEXT",
     "SHOWPLAN_XML",
+    "SID",
     "SINGLE_BLOB",
     "SINGLE_CLOB",
     "SINGLE_NCLOB",
@@ -577,6 +600,7 @@ UNRESERVED_KEYWORDS = [
     "SOURCE",
     "SPARSE",
     "SPATIAL_WINDOW_MAX_CELLS",
+    "SPLIT",
     "START",
     "STATISTICAL_SEMANTICS",
     "STATISTICS_INCREMENTAL",
@@ -609,6 +633,7 @@ UNRESERVED_KEYWORDS = [
     "UNKNOWN",
     "UPDLOCK",
     "USE_TYPE_DEFAULT",
+    "USED",
     "USER_DB",  # Azure Synapse Analytics specific, deprecated
     "USING",
     "VALUE",
@@ -618,11 +643,14 @@ UNRESERVED_KEYWORDS = [
     "WEEK",
     "WEEKS",
     "WHILE",
+    "WINDOWS",
     "WITHIN",
+    "WITHOUT",
     "WITHOUT_ARRAY_WRAPPER",
     "WORK",
     "XACT_ABORT",
     "XLOCK",
+    "XML_COMPRESSION",
     "XML",
     "XMLAGG",  # *future*
     "XMLATTRIBUTES",  # *future*
@@ -645,7 +673,6 @@ UNRESERVED_KEYWORDS = [
     "XMLTABLE",  # *future*
     "XMLTEXT",  # *future*
     "XMLVALIDATE",  # *future*
-    "XML_COMPRESSION",
     "XSINIL",
     "YEAR",
     "YEARS",
