@@ -1,4 +1,5 @@
 """Surrogate class for working with Segment collections."""
+
 from typing import (
     Any,
     Callable,
@@ -6,12 +7,12 @@ from typing import (
     Iterator,
     List,
     Optional,
+    SupportsIndex,
+    Tuple,
     Union,
     overload,
 )
 
-from typing_extensions import SupportsIndex  # NOTE: Required for py37
-
 from sqlfluff.core.parser import BaseSegment
 from sqlfluff.core.templaters.base import TemplatedFile
 from sqlfluff.utils.functional.raw_file_slices import RawFileSlices
@@ -19,14 +20,16 @@ from sqlfluff.utils.functional.raw_file_slices import RawFileSlices
 PredicateType = Callable[[BaseSegment], bool]
 
 
-class Segments(tuple):
+class Segments(Tuple[BaseSegment, ...]):
     """Encapsulates a sequence of one or more BaseSegments.
 
     The segments may or may not be contiguous in a parse tree.
     Provides useful operations on a sequence of segments to simplify rule creation.
     """
 
-    def __new__(cls, *segments, templated_file=None):
+    def __new__(
+        cls, *segments: BaseSegment, templated_file: Optional[TemplatedFile] = None
+    ) -> "Segments":
         """Override new operator."""
         return super(Segments, cls).__new__(cls, segments)
 
@@ -89,7 +92,7 @@ class Segments(tuple):
             )
         return RawFileSlices(
             *sorted(raw_slices, key=lambda slice_: slice_.source_idx),
-            templated_file=self.templated_file
+            templated_file=self.templated_file,
         )
 
     # TODO:This method isn't used as at 2022-08-10. Consider removing in future.
@@ -176,7 +179,9 @@ class Segments(tuple):
         else:
             return result
 
-    def get(self, index: int = 0, *, default: Any = None) -> Optional[BaseSegment]:
+    def get(
+        self, index: int = 0, *, default: Optional[BaseSegment] = None
+    ) -> Optional[BaseSegment]:
         """Return specified item. Returns default if index out of range."""
         try:
             return self[index]
