@@ -2,11 +2,11 @@
 
 import logging
 from itertools import chain
-from typing import Iterator, List, Optional, Sequence, Tuple, Type, cast
+from typing import Iterator, List, Literal, Optional, Sequence, Tuple, Type, cast
 
 from sqlfluff.core.config import FluffConfig
 from sqlfluff.core.parser import BaseSegment, RawSegment
-from sqlfluff.core.rules.base import LintFix, LintResult
+from sqlfluff.core.rules import LintFix, LintResult
 from sqlfluff.utils.reflow.config import ReflowConfig
 from sqlfluff.utils.reflow.depthmap import DepthMap
 from sqlfluff.utils.reflow.elements import (
@@ -16,7 +16,7 @@ from sqlfluff.utils.reflow.elements import (
     get_consumed_whitespace,
 )
 from sqlfluff.utils.reflow.helpers import fixes_from_results
-from sqlfluff.utils.reflow.rebreak import rebreak_sequence
+from sqlfluff.utils.reflow.rebreak import rebreak_keywords_sequence, rebreak_sequence
 from sqlfluff.utils.reflow.reindent import (
     construct_single_indent,
     lint_indent_points,
@@ -156,7 +156,7 @@ class ReflowSequence:
             # Add the block, with config info.
             elem_buff.append(
                 ReflowBlock.from_config(
-                    segments=[seg],
+                    segments=(seg,),
                     config=reflow_config,
                     depth_info=depth_map.get_depth_info(seg),
                 )
@@ -353,7 +353,7 @@ class ReflowSequence:
         # the target.
         self.depth_map.copy_depth_info(target, insertion)
         new_block = ReflowBlock.from_config(
-            segments=[insertion],
+            segments=(insertion,),
             config=self.reflow_config,
             depth_info=self.depth_map.get_depth_info(target),
         )
@@ -503,8 +503,8 @@ class ReflowSequence:
             # If filter has been set, optionally unset the returned values.
             if (
                 filter == "inline"
+                # NOTE: We test on the NEW point.
                 if (
-                    # NOTE: We test on the NEW point.
                     any(seg.is_type("newline") for seg in new_point.segments)
                     # Or if it's followed by the end of file
                     or (post and "end_of_file" in post.class_types)
@@ -533,7 +533,9 @@ class ReflowSequence:
             lint_results=lint_results,
         )
 
-    def rebreak(self) -> "ReflowSequence":
+    def rebreak(
+        self, rebreak_type: Literal["lines", "keywords"] = "lines"
+    ) -> "ReflowSequence":
         """Returns a new :obj:`ReflowSequence` corrected line breaks.
 
         This intentionally **does not handle indentation**,
@@ -552,7 +554,16 @@ class ReflowSequence:
             )
 
         # Delegate to the rebreak algorithm
-        elem_buff, lint_results = rebreak_sequence(self.elements, self.root_segment)
+        if rebreak_type == "lines":
+            elem_buff, lint_results = rebreak_sequence(self.elements, self.root_segment)
+        elif rebreak_type == "keywords":
+            elem_buff, lint_results = rebreak_keywords_sequence(
+                self.elements, self.root_segment
+            )
+        else:  # pragma: no cover
+            raise NotImplementedError(
+                f"Rebreak type of `{rebreak_type}` is not supported."
+            )
 
         return ReflowSequence(
             elements=elem_buff,
@@ -580,6 +591,7 @@ class ReflowSequence:
             single_indent=single_indent,
             skip_indentation_in=self.reflow_config.skip_indentation_in,
             allow_implicit_indents=self.reflow_config.allow_implicit_indents,
+            ignore_comment_lines=self.reflow_config.ignore_comment_lines,
         )
 
         return ReflowSequence(
@@ -590,7 +602,7 @@ class ReflowSequence:
             lint_results=indent_results,
         )
 
-    def break_long_lines(self):
+    def break_long_lines(self) -> "ReflowSequence":
         """Rebreak any remaining long lines in a sequence.
 
         This assumes that reindent() has already been applied.
