@@ -1,4 +1,5 @@
 """Testing utils for working with the CLIs."""
+
 from typing import Any, Dict, List, Optional
 
 from click.testing import CliRunner, Result
@@ -10,7 +11,8 @@ def invoke_assert_code(
     kwargs: Optional[Dict[str, Any]] = None,
     cli_input: Optional[str] = None,
     mix_stderr: bool = True,
-    output_contains: str = "",
+    assert_output_contains: str = "",
+    raise_exceptions: bool = True,
 ) -> Result:
     """Invoke a command and check return code."""
     args = args or []
@@ -21,11 +23,14 @@ def invoke_assert_code(
     result = runner.invoke(*args, **kwargs)
     # Output the CLI code for debugging
     print(result.output)
-    # Check return codes
-    if output_contains != "":
-        assert output_contains in result.output
-    if ret_code == 0:
-        if result.exception:
-            raise result.exception
+    if assert_output_contains != "":
+        # The replace command just accounts for cross platform testing.
+        assert assert_output_contains in result.output.replace("\\", "/")
+    # Check return codes, and unless we specifically want to pass back exceptions,
+    # we should raise any exceptions which aren't `SystemExit` ones (i.e. ones
+    # raised by `sys.exit()`)
+    if raise_exceptions and result.exception:
+        if not isinstance(result.exception, SystemExit):
+            raise result.exception  # pragma: no cover
     assert ret_code == result.exit_code
     return result
