@@ -1,6 +1,6 @@
 """Tests for use cases of the public api classes."""
 
-from sqlfluff.core import Parser, Lexer, Linter
+from sqlfluff.core import Lexer, Linter, Parser
 
 test_query = "SELECt 1"
 
@@ -26,7 +26,7 @@ def test__api__linter_lint():
     tokens, _ = Lexer(dialect="ansi").lex(test_query)
     parsed = Parser(dialect="ansi").parse(tokens)
     violations = Linter(dialect="ansi").lint(parsed)
-    assert [v.rule.code for v in violations] == ["L009", "L010"]
+    assert [v.rule.code for v in violations] == ["CP01", "LT12"]
 
 
 def test__api__linter_fix():
