@@ -1,4 +1,5 @@
 """Common Test Fixtures."""
+
 import hashlib
 import io
 import os
@@ -82,7 +83,7 @@ def make_dialect_path(dialect, fname):
 
 def load_file(dialect, fname):
     """Load a file."""
-    with open(make_dialect_path(dialect, fname)) as f:
+    with open(make_dialect_path(dialect, fname), encoding="utf8") as f:
         raw = f.read()
     return raw
 
@@ -128,7 +129,7 @@ def compute_parse_tree_hash(tree):
         r = tree.as_record(code_only=True, show_raw=True)
         if r:
             r_io = io.StringIO()
-            yaml.dump(r, r_io, sort_keys=False)
+            yaml.dump(r, r_io, sort_keys=False, allow_unicode=True)
             result = hashlib.blake2s(r_io.getvalue().encode("utf-8")).hexdigest()
             return result
     return None
@@ -137,7 +138,7 @@ def compute_parse_tree_hash(tree):
 def load_yaml(fpath):
     """Load a yaml structure and process it into a tuple."""
     # Load raw file
-    with open(fpath) as f:
+    with open(fpath, encoding="utf8") as f:
         raw = f.read()
     # Parse the yaml
     obj = yaml.safe_load(raw)
@@ -300,8 +301,3 @@ def test_verbosity_level(request):
     Has a verbosity level of 0
     """
     return request.config.getoption("verbose")
-
-
-def pytest_addoption(parser):
-    """Allow to run test/rules/yaml_test_cases_test.py for a specific rule_id."""
-    parser.addoption("--rule_id", action="store", default="*", help="Rule id to run")
