@@ -2,7 +2,7 @@
 
 import pytest
 
-from sqlfluff.core.parser.helpers import trim_non_code_segments, iter_indices
+from sqlfluff.core.parser.helpers import trim_non_code_segments
 
 
 @pytest.mark.parametrize(
@@ -22,26 +22,11 @@ def test__parser__helper_trim_non_code_segments(
     generate_test_segments,
 ):
     """Test trim_non_code_segments."""
-    seg_list = generate_test_segments(token_list)
-    pre, mid, post = trim_non_code_segments(seg_list)
+    segments = generate_test_segments(token_list)
+    pre, mid, post = trim_non_code_segments(segments)
     # Assert lengths
     assert (len(pre), len(mid), len(post)) == (pre_len, mid_len, post_len)
     # Assert content
     assert [elem.raw for elem in pre] == list(token_list[:pre_len])
     assert [elem.raw for elem in mid] == list(token_list[pre_len : pre_len + mid_len])
-    assert [elem.raw for elem in post] == list(token_list[len(seg_list) - post_len :])
-
-
-@pytest.mark.parametrize(
-    "seq,val,indices",
-    [
-        ([], 1, []),
-        ([0, 1, 0], 2, []),
-        ([0, 1, 0], 1, [1]),
-        ([0, 1, 0], 0, [0, 2]),
-    ],
-)
-def test__parser__helper_iter_indices(seq, val, indices):
-    """Test iter_indices."""
-    res = list(iter_indices(seq, val))
-    assert res == indices
+    assert [elem.raw for elem in post] == list(token_list[len(segments) - post_len :])
