@@ -8,5 +8,4 @@ LIST FILE "/path/to/some/directory";
 
 LIST FILES "/path with space/cde.txt" '/path with space/fgh.txt';
 
--- NB: Non-quoted paths are not supported in SQLFluff currently
---LIST FILE /tmp/test;
+LIST FILE /tmp/test;
