@@ -59,3 +59,18 @@ GROUP BY age
 HAVING COUNT(age) > 1
 DISTRIBUTE BY age
 SORT BY age;
+
+SELECT
+    age,
+    name
+FROM person
+UNION ALL
+SELECT
+    age,
+    name
+FROM person_cold
+DISTRIBUTE BY age
+SORT BY age;
+
+SELECT CURRENT_DATE() AS p_data_date
+DISTRIBUTE BY p_data_date;
