@@ -6,12 +6,12 @@ FROM test TABLESAMPLE(50 PERCENT);
 SELECT
     t.a,
     t.b
-FROM test t TABLESAMPLE(50 PERCENT);
+FROM test TABLESAMPLE(50 PERCENT) t;
 
 SELECT
     t.a,
     t.b
-FROM test AS t TABLESAMPLE(50 PERCENT);
+FROM test TABLESAMPLE(50 PERCENT) AS t;
 
 SELECT
     a,
@@ -32,11 +32,11 @@ NATURAL JOIN test_2 TABLESAMPLE(BUCKET 4 OUT OF 10);
 SELECT
     t1.a,
     t2.b
-FROM test_1 t1 TABLESAMPLE(5 ROWS)
-NATURAL JOIN test_2 t2 TABLESAMPLE(BUCKET 4 OUT OF 10);
+FROM test_1 TABLESAMPLE(5 ROWS) t1
+NATURAL JOIN test_2 TABLESAMPLE(BUCKET 4 OUT OF 10) t2;
 
 SELECT
     t1.a,
     t2.b
-FROM test_1 AS t1 TABLESAMPLE(5 ROWS)
-NATURAL JOIN test_2 AS t2 TABLESAMPLE(BUCKET 4 OUT OF 10);
+FROM test_1 TABLESAMPLE(5 ROWS) AS t1
+NATURAL JOIN test_2 TABLESAMPLE(BUCKET 4 OUT OF 10) AS t2;
