@@ -23,3 +23,13 @@ SELECT * FROM (SELECT
 INSERT INTO t1 DEFAULT VALUES;
 
 INSERT INTO t1 (a, b, c) DEFAULT VALUES;
+
+INSERT INTO t1 (a, b, c) DEFAULT VALUES RETURNING *;
+
+INSERT INTO t1 (a, b, c) DEFAULT VALUES RETURNING a foo;
+
+INSERT INTO t1 (a, b, c) DEFAULT VALUES RETURNING a AS foo, *;
+
+INSERT INTO t1 (a, b, c) DEFAULT VALUES RETURNING a AS foo;
+
+INSERT INTO users (name, email) VALUES (1, 2) RETURNING *;
