@@ -10,6 +10,10 @@ BEGIN
 
 	DECLARE @EOMONTH DATE = ('1900-01-01')
 
+	DECLARE @USER DATE = SYSTEM_USER;
+
+	DECLARE @CURRENTTIME DATE = CURRENT_TIMESTAMP;
+
 	SET @EOMONTH = ('2000-01-01')
 
 	SET @EOMONTH = ('2001-01-01');
