@@ -1,4 +1,5 @@
 """Utility to generate yml files for all the parsing examples."""
+
 import fnmatch
 import multiprocessing
 import os
@@ -63,6 +64,7 @@ def distribute_work(work_items: List[S], work_fn: Callable[[S], None]) -> None:
                         print(f"  - {fname!r}")
 
     if errors:
+        print(errors)
         print("FAILED TO GENERATE ALL CASES")
         sys.exit(1)
 
@@ -113,7 +115,7 @@ def generate_one_parse_fixture(
     _hash = compute_parse_tree_hash(tree)
     # Remove the .sql file extension
     path = _create_file_path(example)
-    with open(path, "w", newline="\n") as f:
+    with open(path, "w", newline="\n", encoding="utf8") as f:
         r: Optional[Dict[str, Optional[str]]] = None
 
         if not tree:
@@ -135,7 +137,13 @@ def generate_one_parse_fixture(
             file=f,
             sep="\n",
         )
-        yaml.dump(r, f, default_flow_style=False, sort_keys=False)
+        yaml.dump(
+            data=r,
+            stream=f,
+            default_flow_style=False,
+            sort_keys=False,
+            allow_unicode=True,
+        )
         return example, None
 
 
