@@ -28,22 +28,40 @@ def test__rules__std_ST03_multiple_unused_ctes():
         {
             "code": "ST03",
             "description": 'Query defines CTE "cte_1" but does not use it.',
-            "line_no": 3,
-            "line_pos": 5,
             "name": "structure.unused_cte",
+            "warning": False,
+            "fixes": [],
+            "start_line_no": 3,
+            "start_line_pos": 5,
+            "start_file_pos": 14,
+            "end_line_no": 3,
+            "end_line_pos": 10,
+            "end_file_pos": 19,
         },
         {
             "code": "ST03",
             "description": 'Query defines CTE "cte_2" but does not use it.',
-            "line_no": 6,
-            "line_pos": 5,
             "name": "structure.unused_cte",
+            "warning": False,
+            "fixes": [],
+            "start_line_no": 6,
+            "start_line_pos": 5,
+            "start_file_pos": 53,
+            "end_line_no": 6,
+            "end_line_pos": 10,
+            "end_file_pos": 58,
         },
         {
             "code": "ST03",
             "description": 'Query defines CTE "cte_4" but does not use it.',
-            "line_no": 12,
-            "line_pos": 5,
             "name": "structure.unused_cte",
+            "warning": False,
+            "fixes": [],
+            "start_line_no": 12,
+            "start_line_pos": 5,
+            "start_file_pos": 131,
+            "end_line_no": 12,
+            "end_line_pos": 10,
+            "end_file_pos": 136,
         },
     ]
